@@ -9,8 +9,6 @@
 #ifndef __XMLSEC_MSCNG_CRYPTO_H__
 #define __XMLSEC_MSCNG_CRYPTO_H__
 
-#include <windows.h>
-
 #include <xmlsec/exports.h>
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/keys.h>
@@ -250,14 +248,6 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecMSCngTransformRsaOaepEnc11GetKlass(
 #define xmlSecMSCngKeyDataEcId          xmlSecMSCngKeyDataEcGetKlass()
 XMLSEC_CRYPTO_EXPORT xmlSecKeyDataId    xmlSecMSCngKeyDataEcGetKlass(void);
 
-/**
- * xmlSecMSCngKeyDataEcdsaId:
- *
- * Deprecated. The EC key klass.
- */
-#define xmlSecMSCngKeyDataEcdsaId       xmlSecMSCngKeyDataEcId
-XMLSEC_CRYPTO_EXPORT XMLSEC_DEPRECATED  xmlSecKeyDataId xmlSecMSCngKeyDataEcdsaGetKlass(void);
-
 #ifndef XMLSEC_NO_SHA1
 /**
  * xmlSecMSCngTransformEcdsaSha1Id:
