@@ -6,14 +6,14 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 
 #ifndef __XMLSEC_ERROR_HELPERS_H__
 #define __XMLSEC_ERROR_HELPERS_H__
 
 #ifndef XMLSEC_PRIVATE
-#error "private.h file contains private xmlsec definitions and should not be used outside xmlsec or xmlsec-$crypto libraries"
+#error "errors_helpers.h file contains private xmlsec definitions and should not be used outside xmlsec or xmlsec-$crypto libraries"
 #endif /* XMLSEC_PRIVATE */
 
 #include <errno.h>
@@ -390,6 +390,7 @@ extern "C" {
                     xmlSecErrorsSafeString(msg)             \
         )
 
+
 /**
  * xmlSecInvalidStringDataError:
  * @name:               the name of the variable, parameter, etc.
@@ -807,6 +808,26 @@ extern "C" {
         );                                            \
     }
 
+ /**
+  * xmlSecInvalidTransfromError3:
+  * @transform:          the transform.
+  * @msg:                the extra message.
+  * @param1:             the extra message param.
+  * @param2:             the extra message param.
+  *
+  *
+  * Macro. The XMLSec library macro for reporting an invalid transform errors.
+  */
+#define xmlSecInvalidTransfromError3(transform, msg, param1, param2) \
+    {                                                 \
+        xmlSecError(XMLSEC_ERRORS_HERE,               \
+                   (const char*)xmlSecTransformGetName(transform), \
+                   NULL,                              \
+                   XMLSEC_ERRORS_R_INVALID_TRANSFORM, \
+                   (msg), (param1), (param2)          \
+        );                                            \
+    }
+
 /**
  * xmlSecInvalidTransfromStatusError:
  * @transform:          the transform.
