@@ -1,17 +1,15 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * Private/public keys implementation for GCrypt.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:asymkeys
- * @Short_description: Asymmetric keys implementation for GCrypt.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #include "globals.h"
@@ -26,6 +24,7 @@
 #include <xmlsec/keyinfo.h>
 #include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
+#include <xmlsec/private.h>
 
 #include <xmlsec/gcrypt/crypto.h>
 
@@ -54,7 +53,7 @@ struct _xmlSecGCryptAsymKeyDataCtx {
 
 /******************************************************************************
  *
- * GCrypt asym key data (dsa/rsa)
+ * GCrypt asym key data (dsa/rsa/ec)
  *
  *****************************************************************************/
 XMLSEC_KEY_DATA_DECLARE(GCryptAsymKeyData, xmlSecGCryptAsymKeyDataCtx)
@@ -341,8 +340,12 @@ xmlSecGCryptAsymKeyDataGetSize(xmlSecKeyDataPtr data) {
     ctx = xmlSecGCryptAsymKeyDataGetCtx(data);
     xmlSecAssert2(ctx != NULL, 0);
 
-    /* use pub key since it is more often you have it than not */
-    return (ctx->pub_key != NULL) ? gcry_pk_get_nbits(ctx->pub_key) : 0;
+    if(ctx->priv_key != NULL) {
+        return gcry_pk_get_nbits(ctx->priv_key);
+    } else if(ctx->pub_key != NULL) {
+        return gcry_pk_get_nbits(ctx->pub_key);
+    }
+    return(0);
 }
 
 /******************************************************************************
@@ -490,7 +493,7 @@ done:
 #ifndef XMLSEC_NO_DSA
 /**************************************************************************
  *
- * <dsig:DSAKeyValue> processing
+ * &lt;dsig:DSAKeyValue/&gt; processing
  *
  *
  * The DSAKeyValue Element (http://www.w3.org/TR/xmldsig-core/#sec-DSAKeyValue)
@@ -597,7 +600,7 @@ static xmlSecKeyDataKlass xmlSecGCryptKeyDataDsaKlass = {
 
     /* data */
     xmlSecNameDSAKeyValue,
-    xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
                                                 /* xmlSecKeyDataUsage usage; */
     xmlSecHrefDSAKeyValue,                      /* const xmlChar* href; */
     xmlSecNodeDSAKeyValue,                      /* const xmlChar* dataNodeName; */
@@ -1034,7 +1037,7 @@ done:
 #ifndef XMLSEC_NO_RSA
 /**************************************************************************
  *
- * <dsig:RSAKeyValue> processing
+ * &lt;dsig:RSAKeyValue/&gt; processing
  *
  * http://www.w3.org/TR/xmldsig-core/#sec-RSAKeyValue
  * The RSAKeyValue Element
@@ -1111,7 +1114,7 @@ static xmlSecKeyDataKlass xmlSecGCryptKeyDataRsaKlass = {
 
     /* data */
     xmlSecNameRSAKeyValue,
-    xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
                                                 /* xmlSecKeyDataUsage usage; */
     xmlSecHrefRSAKeyValue,                      /* const xmlChar* href; */
     xmlSecNodeRSAKeyValue,                      /* const xmlChar* dataNodeName; */
@@ -1487,3 +1490,457 @@ done:
     return(res);
 }
 #endif /* XMLSEC_NO_RSA */
+
+
+
+#ifndef XMLSEC_NO_EC
+/**************************************************************************
+ *
+ * EC Keys.
+ *
+ *************************************************************************/
+
+static int              xmlSecGCryptKeyDataEcInitialize         (xmlSecKeyDataPtr data);
+static int              xmlSecGCryptKeyDataEcDuplicate          (xmlSecKeyDataPtr dst,
+                                                                 xmlSecKeyDataPtr src);
+static void             xmlSecGCryptKeyDataEcFinalize           (xmlSecKeyDataPtr data);
+
+static xmlSecKeyDataType xmlSecGCryptKeyDataEcGetType           (xmlSecKeyDataPtr data);
+static xmlSecSize       xmlSecGCryptKeyDataEcGetSize            (xmlSecKeyDataPtr data);
+static void             xmlSecGCryptKeyDataEcDebugDump          (xmlSecKeyDataPtr data,
+                                                                 FILE* output);
+static void             xmlSecGCryptKeyDataEcDebugXmlDump       (xmlSecKeyDataPtr data,
+                                                                 FILE* output);
+
+static int              xmlSecGCryptKeyDataEcXmlRead            (xmlSecKeyDataId id,
+                                                                 xmlSecKeyPtr key,
+                                                                 xmlNodePtr node,
+                                                                 xmlSecKeyInfoCtxPtr keyInfoCtx);
+static int              xmlSecGCryptKeyDataEcXmlWrite           (xmlSecKeyDataId id,
+                                                                 xmlSecKeyPtr key,
+                                                                 xmlNodePtr node,
+                                                                 xmlSecKeyInfoCtxPtr keyInfoCtx);
+
+static xmlSecKeyDataPtr xmlSecGCryptKeyDataEcRead               (xmlSecKeyDataId id,
+                                                                 xmlSecKeyValueEcPtr ecValue);
+static int              xmlSecGCryptKeyDataEcWrite              (xmlSecKeyDataId id,
+                                                                 xmlSecKeyDataPtr data,
+                                                                 xmlSecKeyValueEcPtr ecValue);
+
+static xmlSecKeyDataKlass xmlSecGCryptKeyDataEcKlass = {
+    sizeof(xmlSecKeyDataKlass),
+    xmlSecGCryptAsymKeyDataSize,
+
+    /* data */
+    xmlSecNameECKeyValue,
+    xmlSecKeyDataUsageReadFromFile | xmlSecKeyDataUsageKeyValueNode | xmlSecKeyDataUsageRetrievalMethodNodeXml,
+                                                /* xmlSecKeyDataUsage usage; */
+    xmlSecHrefECKeyValue,                       /* const xmlChar* href; */
+    xmlSecNodeECKeyValue,                       /* const xmlChar* dataNodeName; */
+    xmlSecDSig11Ns,                             /* const xmlChar* dataNodeNs; */
+
+    /* constructors/destructor */
+    xmlSecGCryptKeyDataEcInitialize,            /* xmlSecKeyDataInitializeMethod initialize; */
+    xmlSecGCryptKeyDataEcDuplicate,             /* xmlSecKeyDataDuplicateMethod duplicate; */
+    xmlSecGCryptKeyDataEcFinalize,              /* xmlSecKeyDataFinalizeMethod finalize; */
+    NULL,                                       /* xmlSecKeyDataGenerateMethod generate; */
+
+    /* get info */
+    xmlSecGCryptKeyDataEcGetType,               /* xmlSecKeyDataGetTypeMethod getType; */
+    xmlSecGCryptKeyDataEcGetSize,               /* xmlSecKeyDataGetSizeMethod getSize; */
+    NULL,                                       /* xmlSecKeyDataGetIdentifier getIdentifier; */
+
+    /* read/write */
+    xmlSecGCryptKeyDataEcXmlRead,               /* xmlSecKeyDataXmlReadMethod xmlRead; */
+    xmlSecGCryptKeyDataEcXmlWrite,              /* xmlSecKeyDataXmlWriteMethod xmlWrite; */
+    NULL,                                       /* xmlSecKeyDataBinReadMethod binRead; */
+    NULL,                                       /* xmlSecKeyDataBinWriteMethod binWrite; */
+
+    /* debug */
+    xmlSecGCryptKeyDataEcDebugDump,             /* xmlSecKeyDataDebugDumpMethod debugDump; */
+    xmlSecGCryptKeyDataEcDebugXmlDump,          /* xmlSecKeyDataDebugDumpMethod debugXmlDump; */
+
+    /* reserved for the future */
+    NULL,                                       /* void* reserved0; */
+    NULL,                                       /* void* reserved1; */
+};
+
+/**
+ * xmlSecGCryptkeyDataEcGetKlass:
+ *
+ * The GCrypt EC key data klass.
+ *
+ * Returns: pointer to GCrypt EC key data klass.
+ */
+xmlSecKeyDataId
+xmlSecGCryptkeyDataEcGetKlass(void) {
+    return(&xmlSecGCryptKeyDataEcKlass);
+}
+
+/**
+ * xmlSecGCryptKeyDataEcAdoptKey:
+ * @data:               the pointer to EC key data.
+ * @ec_key:            the pointer to GCrypt EC key.
+ *
+ * Sets the value of EC key data.
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
+int
+xmlSecGCryptKeyDataEcAdoptKey(xmlSecKeyDataPtr data, gcry_sexp_t ec_key) {
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId), -1);
+    xmlSecAssert2(ec_key != NULL, -1);
+    return(xmlSecGCryptAsymKeyDataAdoptKey(data, ec_key));
+}
+
+/**
+ * xmlSecGCryptKeyDataEcAdoptKeyPair:
+ * @data:               the pointer to EC key data.
+ * @pub_key:            the pointer to GCrypt EC pub key.
+ * @priv_key:           the pointer to GCrypt EC priv key.
+ *
+ * Sets the value of EC key data.
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
+int
+xmlSecGCryptKeyDataEcAdoptKeyPair(xmlSecKeyDataPtr data, gcry_sexp_t pub_key, gcry_sexp_t priv_key) {
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId), -1);
+    xmlSecAssert2(pub_key != NULL, -1);
+    return(xmlSecGCryptAsymKeyDataAdoptKeyPair(data, pub_key, priv_key));
+}
+
+/**
+ * xmlSecGCryptKeyDataEcGetPublicKey:
+ * @data:               the pointer to EC key data.
+ *
+ * Gets the GCrypt EC public key from EC key data.
+ *
+ * Returns: pointer to GCrypt public EC key or NULL if an error occurs.
+ */
+gcry_sexp_t
+xmlSecGCryptKeyDataEcGetPublicKey(xmlSecKeyDataPtr data) {
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId), NULL);
+    return(xmlSecGCryptAsymKeyDataGetPublicKey(data));
+}
+
+/**
+ * xmlSecGCryptKeyDataEcGetPrivateKey:
+ * @data:               the pointer to EC key data.
+ *
+ * Gets the GCrypt EC private key from EC key data.
+ *
+ * Returns: pointer to GCrypt private EC key or NULL if an error occurs.
+ */
+gcry_sexp_t
+xmlSecGCryptKeyDataEcGetPrivateKey(xmlSecKeyDataPtr data) {
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId), NULL);
+    return(xmlSecGCryptAsymKeyDataGetPrivateKey(data));
+}
+
+static int
+xmlSecGCryptKeyDataEcInitialize(xmlSecKeyDataPtr data) {
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId), -1);
+
+    return(xmlSecGCryptAsymKeyDataInitialize(data));
+}
+
+static int
+xmlSecGCryptKeyDataEcDuplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
+    xmlSecAssert2(xmlSecKeyDataCheckId(dst, xmlSecGCryptKeyDataEcId), -1);
+    xmlSecAssert2(xmlSecKeyDataCheckId(src, xmlSecGCryptKeyDataEcId), -1);
+
+    return(xmlSecGCryptAsymKeyDataDuplicate(dst, src));
+}
+
+static void
+xmlSecGCryptKeyDataEcFinalize(xmlSecKeyDataPtr data) {
+    xmlSecAssert(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId));
+
+    xmlSecGCryptAsymKeyDataFinalize(data);
+}
+
+static xmlSecKeyDataType
+xmlSecGCryptKeyDataEcGetType(xmlSecKeyDataPtr data) {
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId), xmlSecKeyDataTypeUnknown);
+
+    return xmlSecGCryptAsymKeyDataGetType(data);
+}
+
+static xmlSecSize
+xmlSecGCryptKeyDataEcGetSize(xmlSecKeyDataPtr data) {
+    xmlSecGCryptAsymKeyDataCtxPtr ctx;
+    gcry_sexp_t key;
+    unsigned int nbits = 0;
+    const char *curve;
+
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId), 0);
+
+    ctx = xmlSecGCryptAsymKeyDataGetCtx(data);
+    xmlSecAssert2(ctx != NULL, 0);
+
+    if(ctx->priv_key != NULL) {
+        key = ctx->priv_key;
+    } else if(ctx->pub_key != NULL) {
+        key = ctx->pub_key;
+    } else {
+        return(0);
+    }
+
+    curve = gcry_pk_get_curve(key, 0, &nbits);
+    UNREFERENCED_PARAMETER(curve);
+    return(nbits);
+}
+
+static void
+xmlSecGCryptKeyDataEcDebugDump(xmlSecKeyDataPtr data, FILE* output) {
+    xmlSecAssert(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId));
+    xmlSecAssert(output != NULL);
+
+    fprintf(output, "=== ec key: size = " XMLSEC_SIZE_FMT "\n",
+            xmlSecGCryptKeyDataEcGetSize(data));
+}
+
+static void
+xmlSecGCryptKeyDataEcDebugXmlDump(xmlSecKeyDataPtr data, FILE* output) {
+    xmlSecAssert(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId));
+    xmlSecAssert(output != NULL);
+
+    fprintf(output, "<ECKeyValue size=\"" XMLSEC_SIZE_FMT "\" />\n",
+            xmlSecGCryptKeyDataEcGetSize(data));
+}
+
+static int
+xmlSecGCryptKeyDataEcXmlRead(xmlSecKeyDataId id,
+                              xmlSecKeyPtr key,
+                              xmlNodePtr node,
+                              xmlSecKeyInfoCtxPtr keyInfoCtx)
+{
+    xmlSecAssert2(id == xmlSecGCryptKeyDataEcId, -1);
+    return(xmlSecKeyDataEcXmlRead(id, key, node, keyInfoCtx,
+        xmlSecGCryptKeyDataEcRead));
+}
+
+static int
+xmlSecGCryptKeyDataEcXmlWrite(xmlSecKeyDataId id, xmlSecKeyPtr key,
+                                xmlNodePtr node, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+    xmlSecAssert2(id == xmlSecGCryptKeyDataEcId, -1);
+    return(xmlSecKeyDataEcXmlWrite(id, key, node, keyInfoCtx,
+        xmlSecBase64GetDefaultLineSize(), 1, /* add line breaks */
+        xmlSecGCryptKeyDataEcWrite));
+}
+
+
+typedef struct _xmlSecGCryptKeyDataEcCurveOidToName {
+    char curveName[20];
+    xmlChar curveOid[64];
+} xmlSecGCryptKeyDataEcCurveOidToName;
+
+static xmlSecGCryptKeyDataEcCurveOidToName g_xmlSecGCryptKeyDataEcCurveOidToName[] = {
+    { "prime192v1",  "1.2.840.10045.3.1.1" },
+    { "prime256v1",  "1.2.840.10045.3.1.7" },
+    { "secp224r1",   "1.3.132.0.33" },
+    { "secp384r1",   "1.3.132.0.34" },
+    { "secp521r1",   "1.3.132.0.35" }
+};
+
+static const char*
+xmlSecGCryptKeyDataEcCurveGetNameFromOid(const xmlChar * curveOid) {
+    xmlSecSize size = sizeof(g_xmlSecGCryptKeyDataEcCurveOidToName) / sizeof(g_xmlSecGCryptKeyDataEcCurveOidToName[0]);
+
+    xmlSecAssert2(curveOid != NULL, NULL);
+    for(xmlSecSize ii = 0; ii < size; ++ii) {
+        if(xmlStrcmp(curveOid, g_xmlSecGCryptKeyDataEcCurveOidToName[ii].curveOid) == 0) {
+            return(g_xmlSecGCryptKeyDataEcCurveOidToName[ii].curveName);
+        }
+    }
+    return(NULL);
+}
+
+static const xmlChar*
+xmlSecGCryptKeyDataEcCurveGetOidFromName(const char * curveName) {
+    xmlSecSize size = sizeof(g_xmlSecGCryptKeyDataEcCurveOidToName) / sizeof(g_xmlSecGCryptKeyDataEcCurveOidToName[0]);
+
+    xmlSecAssert2(curveName != NULL, NULL);
+    for(xmlSecSize ii = 0; ii < size; ++ii) {
+        if(strcmp(curveName, g_xmlSecGCryptKeyDataEcCurveOidToName[ii].curveName) == 0) {
+            return(g_xmlSecGCryptKeyDataEcCurveOidToName[ii].curveOid);
+        }
+    }
+    return(NULL);
+}
+
+static xmlSecKeyDataPtr
+xmlSecGCryptKeyDataEcRead(xmlSecKeyDataId id, xmlSecKeyValueEcPtr ecValue) {
+    xmlSecKeyDataPtr data = NULL;
+    xmlSecKeyDataPtr res = NULL;
+    gcry_mpi_t pubkey = NULL;
+    gcry_sexp_t s_pub_key = NULL;
+    gcry_error_t err;
+    const char* curveName;
+    int ret;
+
+    xmlSecAssert2(id == xmlSecGCryptKeyDataEcId, NULL);
+    xmlSecAssert2(ecValue != NULL, NULL);
+    xmlSecAssert2(ecValue->curve != NULL, NULL);
+
+    /* get curve name */
+    curveName = xmlSecGCryptKeyDataEcCurveGetNameFromOid(ecValue->curve);
+    if(curveName == NULL) {
+        xmlSecInternalError2("xmlSecGCryptKeyDataEcCurveGetNameFromOid",  xmlSecKeyDataGetName(data),
+            "curveOid=%s", xmlSecErrorsSafeString(ecValue->curve));
+        goto done;
+    }
+
+    /* pubkey */
+    err = gcry_mpi_scan(&pubkey, GCRYMPI_FMT_USG,
+        xmlSecBufferGetData(&(ecValue->pubkey)), xmlSecBufferGetSize(&(ecValue->pubkey)),
+        NULL);
+    if((err != GPG_ERR_NO_ERROR) || (pubkey == NULL)) {
+        xmlSecGCryptError("gcry_mpi_scan(pubkey)", err, xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    /* construct pub key */
+    err = gcry_sexp_build (&s_pub_key, NULL,
+        "(public-key"
+        " (ecdsa"
+        " (curve %s)"
+        " (q %m)"
+        " ))",
+        curveName, pubkey
+    );
+    if((err != GPG_ERR_NO_ERROR) || (s_pub_key == NULL)) {
+        xmlSecGCryptError("gcry_sexp_build(public)", err, xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    /* create key data */
+    data = xmlSecKeyDataCreate(id);
+    if(data == NULL ) {
+        xmlSecInternalError("xmlSecKeyDataCreate", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    ret = xmlSecGCryptKeyDataEcAdoptKeyPair(data, s_pub_key, NULL);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecGCryptKeyDataEcAdoptKeyPair", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+    s_pub_key = NULL; /* pub_key is owned by data now */
+
+    /* success */
+    res = data;
+    data = NULL;
+
+done:
+    /* cleanup */
+    if(pubkey != NULL) {
+        gcry_mpi_release(pubkey);
+    }
+    if(s_pub_key != NULL) {
+        gcry_sexp_release(s_pub_key);
+    }
+    if(data != NULL) {
+        xmlSecKeyDataDestroy(data);
+    }
+    return(res);
+}
+
+/* use gcry_free() to deallocate returned string */
+static char*
+xmlSecGCryptGetStringFromSExp(const gcry_sexp_t sexp, const char * tok)
+{
+    gcry_sexp_t val = NULL;
+    char* res = NULL;
+
+    xmlSecAssert2(sexp != NULL, NULL);
+    xmlSecAssert2(tok != NULL, NULL);
+
+    val = gcry_sexp_find_token(sexp, tok, 0);
+    if(val == NULL) {
+        xmlSecGCryptError2("gcry_sexp_find_token", (gcry_error_t)GPG_ERR_NO_ERROR, NULL,
+            "tok=%s", xmlSecErrorsSafeString(tok));
+        goto done;
+    }
+
+    res = gcry_sexp_nth_string(val, 1);
+    if(res == NULL) {
+        xmlSecGCryptError2("gcry_sexp_nth_string", (gcry_error_t)GPG_ERR_NO_ERROR, NULL,
+            "tok=%s", xmlSecErrorsSafeString(tok));
+        goto done;
+    }
+
+done:
+    return(res);
+}
+
+
+static int
+xmlSecGCryptKeyDataEcWrite(xmlSecKeyDataId id, xmlSecKeyDataPtr data, xmlSecKeyValueEcPtr ecValue) {
+    gcry_sexp_t s_pub_key;
+    gcry_sexp_t s_ecdsa = NULL;
+    char* curveName = NULL;
+    const xmlChar* curveOid;
+    int res = -1;
+    int ret;
+
+    xmlSecAssert2(id == xmlSecGCryptKeyDataEcId, -1);
+    xmlSecAssert2(data != NULL, -1);
+    xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId), -1);
+    xmlSecAssert2(ecValue != NULL, -1);
+
+    /* find the private or public key */
+    s_pub_key = xmlSecGCryptKeyDataEcGetPublicKey(data);
+    if(s_pub_key == NULL) {
+        xmlSecInternalError("xmlSecGCryptKeyDataEcGetPublicKey()", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    s_ecdsa = gcry_sexp_find_token(s_pub_key, "ecdsa", 0);
+    if(s_ecdsa == NULL) {
+        xmlSecGCryptError("gcry_sexp_find_token(ecdsa)", (gcry_error_t)GPG_ERR_NO_ERROR,
+            xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    /* curve */
+    curveName = xmlSecGCryptGetStringFromSExp(s_ecdsa,  "curve");
+    if(curveName == NULL) {
+        xmlSecInternalError("xmlSecGCryptGetStringFromSExp(curve)", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+    curveOid = xmlSecGCryptKeyDataEcCurveGetOidFromName(curveName);
+    if(curveOid == NULL) {
+        xmlSecInternalError2("xmlSecGCryptKeyDataEcCurveGetNameFromOid",  xmlSecKeyDataKlassGetName(id),
+            "curveName=%s", xmlSecErrorsSafeString(curveName));
+        goto done;
+    }
+    ecValue->curve = xmlStrdup(curveOid);
+    if(ecValue->curve == NULL) {
+        xmlSecStrdupError(curveOid, xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    /* pubkey */
+    ret = xmlSecGCryptSetSExpTokValue(s_ecdsa, "q", &(ecValue->pubkey));
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecGCryptSetSExpTokValue(q)", xmlSecKeyDataKlassGetName(id));
+        goto done;
+    }
+
+    /* success */
+    res = 0;
+
+done:
+    if(curveName != NULL) {
+        gcry_free(curveName);
+    }
+    if(s_ecdsa != NULL) {
+        gcry_sexp_release(s_ecdsa);
+    }
+
+    return(res);
+}
+
+#endif /* XMLSEC_NO_EC */
