@@ -1,17 +1,15 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * HMAC transforms implementation for GCrypt.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:hmac
- * @Short_description: HMAC transforms implementation for GCrypt.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #ifndef XMLSEC_NO_HMAC
@@ -33,43 +31,6 @@
 #include "../keysdata_helpers.h"
 #include "../transform_helpers.h"
 
-/**************************************************************************
- *
- * Configuration
- *
- *****************************************************************************/
-
-/**
- * xmlSecGCryptHmacGetMinOutputLength:
- *
- * DEPRECATED (use @xmlSecTransformHmacGetMinOutputBitsSize instead).
- * Gets the value of min HMAC length.
- *
- * Returns: the min HMAC output length
- */
-int xmlSecGCryptHmacGetMinOutputLength(void)
-{
-    xmlSecSize val = xmlSecTransformHmacGetMinOutputBitsSize();
-    int res;
-
-    XMLSEC_SAFE_CAST_SIZE_TO_INT(val, res, return(-1), NULL);
-    return res;
-}
-
-/**
- * xmlSecGCryptHmacSetMinOutputLength:
- * @min_length: the new min length
- *
- * DEPRECATED (use @xmlSecTransformHmacSetMinOutputBitsSize instead).
- * Sets the min HMAC output length
- */
-void xmlSecGCryptHmacSetMinOutputLength(int min_length)
-{
-    xmlSecSize val;
-    XMLSEC_SAFE_CAST_INT_TO_SIZE(min_length, val, return, NULL);
-    xmlSecTransformHmacSetMinOutputBitsSize(val);
-}
-
 /**************************************************************************
  *
  * Internal GCRYPT HMAC CTX
@@ -80,7 +41,7 @@ struct _xmlSecGCryptHmacCtx {
     int                 digest;
     gcry_md_hd_t        digestCtx;
     xmlSecByte          dgst[XMLSEC_TRASNFORM_HMAC_MAX_OUTPUT_SIZE];
-    xmlSecSize          dgstSize;       /* dgst size in bits */
+    xmlSecSize          dgstSizeInBits;       /* dgst size in bits */
 };
 
 /******************************************************************************
@@ -163,6 +124,7 @@ xmlSecGCryptHmacCheckId(xmlSecTransformPtr transform) {
 static int
 xmlSecGCryptHmacInitialize(xmlSecTransformPtr transform) {
     xmlSecGCryptHmacCtxPtr ctx;
+    xmlSecSize hmacSize;
     gcry_error_t err;
 
     xmlSecAssert2(xmlSecGCryptHmacCheckId(transform), -1);
@@ -215,6 +177,11 @@ xmlSecGCryptHmacInitialize(xmlSecTransformPtr transform) {
         return(-1);
     }
 
+    hmacSize = gcry_md_get_algo_dlen(ctx->digest);
+    xmlSecAssert2(hmacSize > 0, -1);
+    xmlSecAssert2(hmacSize <= XMLSEC_TRASNFORM_HMAC_MAX_OUTPUT_SIZE, -1);
+    ctx->dgstSizeInBits = 8 * hmacSize;
+
     /* open context */
     err = gcry_md_open(&ctx->digestCtx, ctx->digest, GCRY_MD_FLAG_HMAC | GCRY_MD_FLAG_SECURE); /* we are paranoid */
     if(err != GPG_ERR_NO_ERROR) {
@@ -242,9 +209,6 @@ xmlSecGCryptHmacFinalize(xmlSecTransformPtr transform) {
     memset(ctx, 0, sizeof(xmlSecGCryptHmacCtx));
 }
 
-/**
- * xmlSecGCryptHmacNodeRead:
- */
 static int
 xmlSecGCryptHmacNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
                          xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
@@ -259,12 +223,14 @@ xmlSecGCryptHmacNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
     ctx = xmlSecGCryptHmacGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
 
-    ret = xmlSecTransformHmacReadOutputBitsSize(node, ctx->dgstSize, &ctx->dgstSize);
+    ret = xmlSecTransformHmacReadOutputBitsSize(node, ctx->dgstSizeInBits, &ctx->dgstSizeInBits);
     if (ret < 0) {
         xmlSecInternalError("xmlSecTransformHmacReadOutputBitsSize()",
             xmlSecTransformGetName(transform));
         return(-1);
     }
+    xmlSecAssert2(ctx->dgstSizeInBits > 0, -1);
+    xmlSecAssert2(XMLSEC_TRASNFORM_HMAC_BITS_TO_BYTES(ctx->dgstSizeInBits) < XMLSEC_TRASNFORM_HMAC_MAX_OUTPUT_SIZE, -1);
 
     return(0);
 }
@@ -334,11 +300,8 @@ static int
 xmlSecGCryptHmacVerify(xmlSecTransformPtr transform,
                         const xmlSecByte* data, xmlSecSize dataSize,
                         xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
-    static xmlSecByte last_byte_masks[] =
-                { 0xFF, 0x80, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE };
-
     xmlSecGCryptHmacCtxPtr ctx;
-    xmlSecByte mask;
+    int ret;
 
     xmlSecAssert2(xmlSecTransformIsValid(transform), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecGCryptHmacSize), -1);
@@ -350,38 +313,21 @@ xmlSecGCryptHmacVerify(xmlSecTransformPtr transform,
     ctx = xmlSecGCryptHmacGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
     xmlSecAssert2(ctx->digestCtx != NULL, -1);
-    xmlSecAssert2(ctx->dgstSize > 0, -1);
+    xmlSecAssert2(ctx->dgstSizeInBits > 0, -1);
 
-    /* compare the digest size in bytes */
-    if(dataSize != ((ctx->dgstSize + 7) / 8)) {
-        xmlSecInvalidSizeError("HMAC digest size",
-                              dataSize, ((ctx->dgstSize + 7) / 8),
-                              xmlSecTransformGetName(transform));
+    /* Returns 1 for match, 0 for no match, <0 for errors. */
+    ret = xmlSecTransformHmacVerify(data, dataSize, ctx->dgst, ctx->dgstSizeInBits, sizeof(ctx->dgst));
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecTransformHmacVerify", xmlSecTransformGetName(transform));
+        return(-1);
+    }
+    if(ret == 1) {
+        transform->status = xmlSecTransformStatusOk;
+    } else {
         transform->status = xmlSecTransformStatusFail;
-        return(0);
     }
 
-    /* we check the last byte separately */
-    xmlSecAssert2(dataSize > 0, -1);
-    mask = last_byte_masks[ctx->dgstSize % 8];
-    if((ctx->dgst[dataSize - 1] & mask) != (data[dataSize - 1]  & mask)) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_DATA_NOT_MATCH,
-                         xmlSecTransformGetName(transform),
-                         "data and digest do not match (last byte)");
-        transform->status = xmlSecTransformStatusFail;
-        return(0);
-    }
-
-    /* now check the rest of the digest */
-    if((dataSize > 1) && (memcmp(ctx->dgst, data, dataSize - 1) != 0)) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_DATA_NOT_MATCH,
-                         xmlSecTransformGetName(transform),
-                         "data and digest do not match");
-        transform->status = xmlSecTransformStatusFail;
-        return(0);
-    }
-
-    transform->status = xmlSecTransformStatusOk;
+    /* done */
     return(0);
 }
 
@@ -390,7 +336,6 @@ xmlSecGCryptHmacExecute(xmlSecTransformPtr transform, int last, xmlSecTransformC
     xmlSecGCryptHmacCtxPtr ctx;
     xmlSecBufferPtr in, out;
     xmlSecByte* dgst;
-    xmlSecSize dgstSize;
     int ret;
 
     xmlSecAssert2(xmlSecGCryptHmacCheckId(transform), -1);
@@ -433,31 +378,13 @@ xmlSecGCryptHmacExecute(xmlSecTransformPtr transform, int last, xmlSecTransformC
                                   xmlSecTransformGetName(transform));
                 return(-1);
             }
+            memcpy(ctx->dgst, dgst, XMLSEC_TRASNFORM_HMAC_BITS_TO_BYTES(ctx->dgstSizeInBits));
 
-            /* copy it to our internal buffer */
-            dgstSize = gcry_md_get_algo_dlen(ctx->digest);
-            xmlSecAssert2(dgstSize > 0, -1);
-            xmlSecAssert2(dgstSize <= sizeof(ctx->dgst), -1);
-            memcpy(ctx->dgst, dgst, dgstSize);
-
-            /* check/set the result digest size */
-            if(ctx->dgstSize == 0) {
-                ctx->dgstSize = dgstSize * 8; /* no dgst size specified, use all we have */
-            } else if(ctx->dgstSize <= 8 * dgstSize) {
-                xmlSecSize adjustedDigestSize = ((ctx->dgstSize + 7) / 8); /* we need to truncate result digest */
-                XMLSEC_SAFE_CAST_SIZE_TO_UINT(adjustedDigestSize, dgstSize, return(-1), xmlSecTransformGetName(transform));
-            } else {
-                xmlSecInvalidSizeLessThanError("HMAC digest (bits)",
-                    8 * dgstSize, ctx->dgstSize,
-                    xmlSecTransformGetName(transform));
-                return(-1);
-            }
-
+            /* write results if needed */
             if(transform->operation == xmlSecTransformOperationSign) {
-                ret = xmlSecBufferAppend(out, ctx->dgst, dgstSize);
+                ret = xmlSecTransformHmacWriteOutput(ctx->dgst, ctx->dgstSizeInBits, XMLSEC_TRASNFORM_HMAC_MAX_OUTPUT_SIZE, out);
                 if(ret < 0) {
-                    xmlSecInternalError2("xmlSecBufferAppend", xmlSecTransformGetName(transform),
-                        "size=" XMLSEC_SIZE_FMT, dgstSize);
+                    xmlSecInternalError("xmlSecTransformHmacWriteOutput", xmlSecTransformGetName(transform));
                     return(-1);
                 }
             }
