@@ -5,16 +5,12 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:kw_des
- * @Short_description: DES Key Transport transforms implementation for GCrypt.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
-
 #ifndef XMLSEC_NO_DES
 #include "globals.h"
 
@@ -466,6 +462,9 @@ xmlSecGCryptKWDes3Encrypt(const xmlSecByte *key, xmlSecSize keySize,
     return(0);
 }
 
+#else /* XMLSEC_NO_DES */
+
+/* ISO C forbids an empty translation unit */
+typedef int make_iso_compilers_happy;
 
 #endif /* XMLSEC_NO_DES */
-
