@@ -61,6 +61,7 @@
 #include <xmlsec/keys.h>
 #include <xmlsec/keysmngr.h>
 #include <xmlsec/transforms.h>
+#include <xmlsec/parser.h>
 #include <xmlsec/xmlenc.h>
 #include <xmlsec/keyinfo.h>
 #include <xmlsec/errors.h>
@@ -107,12 +108,12 @@ xmlSecKeyInfoNodeRead(xmlNodePtr keyInfoNode, xmlSecKeyPtr key, xmlSecKeyInfoCtx
         nodeName = cur->name;
         nodeNs = xmlSecGetNodeNsHref(cur);
 
-        /* use global list only if we don't have a local one */
+        /* use global enabled list only if we don't have a local one */
         if(xmlSecPtrListGetSize(&(keyInfoCtx->enabledKeyData)) > 0) {
             dataId = xmlSecKeyDataIdListFindByNode(&(keyInfoCtx->enabledKeyData),
                             nodeName, nodeNs, xmlSecKeyDataUsageKeyInfoNodeRead);
         } else {
-            dataId = xmlSecKeyDataIdListFindByNode(xmlSecKeyDataIdsGet(),
+            dataId = xmlSecKeyDataIdListFindByNode(xmlSecKeyDataIdsGetEnabled(),
                             nodeName, nodeNs, xmlSecKeyDataUsageKeyInfoNodeRead);
         }
         if(dataId != xmlSecKeyDataIdUnknown) {
@@ -130,6 +131,8 @@ xmlSecKeyInfoNodeRead(xmlNodePtr keyInfoNode, xmlSecKeyPtr key, xmlSecKeyInfoCtx
              * desire to disable unknown nodes*/
             xmlSecUnexpectedNodeError(cur, NULL);
             return(-1);
+        } else {
+            xmlSecUnexpectedNodeError(cur, NULL);
         }
     }
 
@@ -167,13 +170,13 @@ xmlSecKeyInfoNodeWrite(xmlNodePtr keyInfoNode, xmlSecKeyPtr key, xmlSecKeyInfoCt
         nodeName = cur->name;
         nodeNs = xmlSecGetNodeNsHref(cur);
 
-        /* use global list only if we don't have a local one */
+        /* use global eanbled list only if we don't have a local one */
         if(xmlSecPtrListGetSize(&(keyInfoCtx->enabledKeyData)) > 0) {
                 dataId = xmlSecKeyDataIdListFindByNode(&(keyInfoCtx->enabledKeyData),
                             nodeName, nodeNs,
                             xmlSecKeyDataUsageKeyInfoNodeWrite);
         } else {
-                dataId = xmlSecKeyDataIdListFindByNode(xmlSecKeyDataIdsGet(),
+                dataId = xmlSecKeyDataIdListFindByNode(xmlSecKeyDataIdsGetEnabled(),
                             nodeName, nodeNs,
                             xmlSecKeyDataUsageKeyInfoNodeWrite);
         }
@@ -862,12 +865,12 @@ xmlSecKeyDataValueXmlRead(xmlSecKeyDataId id, xmlSecKeyPtr key, xmlNodePtr node,
     nodeName = cur->name;
     nodeNs = xmlSecGetNodeNsHref(cur);
 
-    /* use global list only if we don't have a local one */
+    /* use global enabled list only if we don't have a local one */
     if(xmlSecPtrListGetSize(&(keyInfoCtx->enabledKeyData)) > 0) {
         dataId = xmlSecKeyDataIdListFindByNode(&(keyInfoCtx->enabledKeyData),
                             nodeName, nodeNs, xmlSecKeyDataUsageKeyValueNodeRead);
     } else {
-        dataId = xmlSecKeyDataIdListFindByNode(xmlSecKeyDataIdsGet(),
+        dataId = xmlSecKeyDataIdListFindByNode(xmlSecKeyDataIdsGetEnabled(),
                             nodeName, nodeNs, xmlSecKeyDataUsageKeyValueNodeRead);
     }
     if(dataId != xmlSecKeyDataIdUnknown) {
@@ -1062,12 +1065,12 @@ xmlSecKeyDataRetrievalMethodXmlRead(xmlSecKeyDataId id, xmlSecKeyPtr key, xmlNod
 
     retrType = xmlGetProp(node, xmlSecAttrType);
     if(retrType != NULL) {
-        /* use global list only if we don't have a local one */
+        /* use global enabled list only if we don't have a local one */
         if(xmlSecPtrListGetSize(&(keyInfoCtx->enabledKeyData)) > 0) {
             dataId = xmlSecKeyDataIdListFindByHref(&(keyInfoCtx->enabledKeyData),
                             retrType, xmlSecKeyDataUsageRetrievalMethodNode);
         } else {
-            dataId = xmlSecKeyDataIdListFindByHref(xmlSecKeyDataIdsGet(),
+            dataId = xmlSecKeyDataIdListFindByHref(xmlSecKeyDataIdsGetEnabled(),
                             retrType, xmlSecKeyDataUsageRetrievalMethodNode);
         }
     }
@@ -1197,9 +1200,9 @@ xmlSecKeyDataRetrievalMethodReadXmlResult(xmlSecKeyDataId typeId, xmlSecKeyPtr k
     xmlSecAssert2(keyInfoCtx->mode == xmlSecKeyInfoModeRead, -1);
 
     XMLSEC_SAFE_CAST_SIZE_TO_INT(bufferSize, bufferLen, return(-1), NULL);
-    doc = xmlRecoverMemory((const char*)buffer, bufferLen);
+    doc = xmlReadMemory((const char*)buffer, bufferLen, NULL, NULL, xmlSecParserGetDefaultOptions() | XML_PARSE_RECOVER);
     if(doc == NULL) {
-        xmlSecXmlError("xmlRecoverMemory", xmlSecKeyDataKlassGetName(typeId));
+        xmlSecXmlError("xmlReadMemory", xmlSecKeyDataKlassGetName(typeId));
         return(-1);
     }
 
@@ -1213,12 +1216,12 @@ xmlSecKeyDataRetrievalMethodReadXmlResult(xmlSecKeyDataId typeId, xmlSecKeyPtr k
     nodeName = cur->name;
     nodeNs = xmlSecGetNodeNsHref(cur);
 
-    /* use global list only if we don't have a local one */
+    /* use global enabled list only if we don't have a local one */
     if(xmlSecPtrListGetSize(&(keyInfoCtx->enabledKeyData)) > 0) {
         dataId = xmlSecKeyDataIdListFindByNode(&(keyInfoCtx->enabledKeyData),
                             nodeName, nodeNs, xmlSecKeyDataUsageRetrievalMethodNodeXml);
     } else {
-        dataId = xmlSecKeyDataIdListFindByNode(xmlSecKeyDataIdsGet(),
+        dataId = xmlSecKeyDataIdListFindByNode(xmlSecKeyDataIdsGetEnabled(),
                             nodeName, nodeNs, xmlSecKeyDataUsageRetrievalMethodNodeXml);
     }
     if(dataId == xmlSecKeyDataIdUnknown) {
@@ -1452,9 +1455,9 @@ xmlSecKeyDataKeyInfoReferenceReadXmlResult(xmlSecKeyDataId typeId, xmlSecKeyPtr
 
 
     XMLSEC_SAFE_CAST_SIZE_TO_INT(bufferSize, bufferLen, return(-1), NULL);
-    doc = xmlRecoverMemory((const char*)buffer, bufferLen);
+    doc = xmlReadMemory((const char*)buffer, bufferLen, NULL, NULL, xmlSecParserGetDefaultOptions() | XML_PARSE_RECOVER);
     if(doc == NULL) {
-        xmlSecXmlError("xmlRecoverMemory", xmlSecKeyDataKlassGetName(typeId));
+        xmlSecXmlError("xmlReadMemory", xmlSecKeyDataKlassGetName(typeId));
         return(-1);
     }
 
