@@ -5,7 +5,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
  * SECTION:keysmngr
@@ -96,7 +96,7 @@ xmlSecKeysMngrDestroy(xmlSecKeysMngrPtr mngr) {
  * xmlSecKeysMngrFindKey:
  * @mngr:               the pointer to keys manager.
  * @name:               the desired key name.
- * @keyInfoCtx:         the pointer to <dsig:KeyInfo/> node processing context.
+ * @keyInfoCtx:         the pointer to &lt;dsig:KeyInfo/&gt; node processing context.
  *
  * Lookups key in the keys manager keys store. The caller is responsible
  * for destroying the returned key using #xmlSecKeyDestroy method.
@@ -119,6 +119,35 @@ xmlSecKeysMngrFindKey(xmlSecKeysMngrPtr mngr, const xmlChar* name, xmlSecKeyInfo
     return(xmlSecKeyStoreFindKey(store, name, keyInfoCtx));
 }
 
+/**
+ * xmlSecKeysMngrFindKeyFromX509Data:
+ * @mngr:               the pointer to keys manager.
+ * @x509Data:           the X509 data to use for searching the keys.
+ * @keyInfoCtx:         the pointer to &lt;dsig:KeyInfo/&gt; node processing context.
+ *
+ * Lookups key in the keys manager keys store. The caller is responsible
+ * for destroying the returned key using #xmlSecKeyDestroy method.
+ *
+ * Returns: the pointer to a key or NULL if key is not found or an error occurs.
+ */
+xmlSecKeyPtr
+xmlSecKeysMngrFindKeyFromX509Data(xmlSecKeysMngrPtr mngr, xmlSecKeyX509DataValuePtr x509Data, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+    xmlSecKeyStorePtr store;
+
+    xmlSecAssert2(mngr != NULL, NULL);
+    xmlSecAssert2(x509Data != NULL, NULL);
+    xmlSecAssert2(keyInfoCtx != NULL, NULL);
+
+    store = xmlSecKeysMngrGetKeysStore(mngr);
+    if(store == NULL) {
+        /* no store. is it an error? */
+        return(NULL);
+    }
+
+    return(xmlSecKeyStoreFindKeyFromX509Data(store, x509Data, keyInfoCtx));
+}
+
+
 /**
  * xmlSecKeysMngrAdoptKeysStore:
  * @mngr:               the pointer to keys manager.
@@ -282,7 +311,7 @@ xmlSecKeyStoreDestroy(xmlSecKeyStorePtr store) {
  * xmlSecKeyStoreFindKey:
  * @store:              the pointer to keys store.
  * @name:               the desired key name.
- * @keyInfoCtx:         the pointer to <dsig:KeyInfo/> node processing context.
+ * @keyInfoCtx:         the pointer to &lt;dsig:KeyInfo/&gt; node processing context.
  *
  * Lookups key in the store. The caller is responsible for destroying
  * the returned key using #xmlSecKeyDestroy method.
@@ -292,12 +321,37 @@ xmlSecKeyStoreDestroy(xmlSecKeyStorePtr store) {
 xmlSecKeyPtr
 xmlSecKeyStoreFindKey(xmlSecKeyStorePtr store, const xmlChar* name, xmlSecKeyInfoCtxPtr keyInfoCtx) {
     xmlSecAssert2(xmlSecKeyStoreIsValid(store), NULL);
-    xmlSecAssert2(store->id->findKey != NULL, NULL);
     xmlSecAssert2(keyInfoCtx != NULL, NULL);
 
+    if(store->id->findKey == NULL) {
+        return(NULL);
+    }
     return(store->id->findKey(store, name, keyInfoCtx));
 }
 
+/**
+ * xmlSecKeyStoreFindKeyFromX509Data:
+ * @store:              the pointer to keys store.
+ * @x509Data:           the X509 data to use for search.
+ * @keyInfoCtx:         the pointer to &lt;dsig:KeyInfo/&gt; node processing context.
+ *
+ * Lookups key in the store. The caller is responsible for destroying
+ * the returned key using #xmlSecKeyDestroy method.
+ *
+ * Returns: the pointer to a key or NULL if key is not found or an error occurs.
+ */
+xmlSecKeyPtr
+xmlSecKeyStoreFindKeyFromX509Data(xmlSecKeyStorePtr store, xmlSecKeyX509DataValuePtr x509Data, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+    xmlSecAssert2(xmlSecKeyStoreIsValid(store), NULL);
+    xmlSecAssert2(x509Data != NULL, NULL);
+    xmlSecAssert2(keyInfoCtx != NULL, NULL);
+
+    if(store->id->findKeyFromX509Data == NULL) {
+        return(NULL);
+    }
+    return(store->id->findKeyFromX509Data(store, x509Data, keyInfoCtx));
+}
+
 /****************************************************************************
  *
  * Simple Keys Store
@@ -325,10 +379,10 @@ static xmlSecKeyStoreKlass xmlSecSimpleKeysStoreKlass = {
     xmlSecSimpleKeysStoreInitialize,            /* xmlSecKeyStoreInitializeMethod initialize; */
     xmlSecSimpleKeysStoreFinalize,              /* xmlSecKeyStoreFinalizeMethod finalize; */
     xmlSecSimpleKeysStoreFindKey,               /* xmlSecKeyStoreFindKeyMethod findKey; */
+    NULL,                                       /* xmlSecKeyStoreFindKeyFromX509DataMethod findKeyFromX509Data; */
 
     /* reserved for the future */
     NULL,                                       /* void* reserved0; */
-    NULL,                                       /* void* reserved1; */
 };
 
 /**
@@ -392,6 +446,34 @@ xmlSecSimpleKeysStoreLoad(xmlSecKeyStorePtr store, const char *uri,
         xmlSecSimpleKeysStoreAdoptKey));
 }
 
+static int
+xmlSecSimpleKeysStoreEnableAllKeyData(xmlSecKeyInfoCtxPtr keyInfoCtx) {
+    xmlSecPtrListPtr list;
+    xmlSecSize ii, size;
+    xmlSecKeyDataId dataId;
+    int ret;
+
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+
+    list = xmlSecKeyDataIdsGet();
+    xmlSecAssert2(list != NULL, -1);
+
+    size = xmlSecPtrListGetSize(list);
+    for(ii = 0; ii < size; ++ii) {
+        dataId = (xmlSecKeyDataId)xmlSecPtrListGetItem(list, ii);
+        xmlSecAssert2(dataId != xmlSecKeyDataIdUnknown, -1);
+
+        ret = xmlSecPtrListAdd(&(keyInfoCtx->enabledKeyData), (const xmlSecPtr)dataId);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecPtrListAdd", NULL);
+            return(-1);
+        }
+    }
+
+    /* done */
+    return(0);
+}
+
 /**
  * xmlSecSimpleKeysStoreLoad_ex:
  * @store:              the pointer to simple keys store.
@@ -429,9 +511,8 @@ xmlSecSimpleKeysStoreLoad_ex(xmlSecKeyStorePtr store, const char *uri,
     }
 
     root = xmlDocGetRootElement(doc);
-    if(!xmlSecCheckNodeName(root, BAD_CAST "Keys", xmlSecNs)) {
-        xmlSecInvalidNodeError(root, BAD_CAST "Keys",
-            xmlSecKeyStoreGetName(store));
+    if((root == NULL) || (!xmlSecCheckNodeName(root, BAD_CAST "Keys", xmlSecNs))) {
+        xmlSecInvalidNodeError(root, BAD_CAST "Keys", xmlSecKeyStoreGetName(store));
         xmlFreeDoc(doc);
         return(-1);
     }
@@ -440,16 +521,14 @@ xmlSecSimpleKeysStoreLoad_ex(xmlSecKeyStorePtr store, const char *uri,
     while((cur != NULL) && xmlSecCheckNodeName(cur, xmlSecNodeKeyInfo, xmlSecDSigNs)) {
         key = xmlSecKeyCreate();
         if(key == NULL) {
-            xmlSecInternalError("xmlSecKeyCreate",
-                xmlSecKeyStoreGetName(store));
+            xmlSecInternalError("xmlSecKeyCreate", xmlSecKeyStoreGetName(store));
             xmlFreeDoc(doc);
             return(-1);
         }
 
         ret = xmlSecKeyInfoCtxInitialize(&keyInfoCtx, NULL);
         if(ret < 0) {
-            xmlSecInternalError("xmlSecKeyInfoCtxInitialize",
-                xmlSecKeyStoreGetName(store));
+            xmlSecInternalError("xmlSecKeyInfoCtxInitialize", xmlSecKeyStoreGetName(store));
             xmlSecKeyDestroy(key);
             xmlFreeDoc(doc);
             return(-1);
@@ -463,10 +542,19 @@ xmlSecSimpleKeysStoreLoad_ex(xmlSecKeyStorePtr store, const char *uri,
         keyInfoCtx.keyReq.keyType = xmlSecKeyDataTypeAny;
         keyInfoCtx.keyReq.keyUsage= xmlSecKeyDataUsageAny;
 
+        /* enable all keydata for store */
+        ret = xmlSecSimpleKeysStoreEnableAllKeyData(&keyInfoCtx);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecSimpleKeysStoreEnableAllKeyData", xmlSecKeyStoreGetName(store));
+            xmlSecKeyInfoCtxFinalize(&keyInfoCtx);
+            xmlSecKeyDestroy(key);
+            xmlFreeDoc(doc);
+            return(-1);
+        }
+
         ret = xmlSecKeyInfoNodeRead(cur, key, &keyInfoCtx);
         if(ret < 0) {
-            xmlSecInternalError("xmlSecKeyInfoNodeRead",
-                xmlSecKeyStoreGetName(store));
+            xmlSecInternalError("xmlSecKeyInfoNodeRead", xmlSecKeyStoreGetName(store));
             xmlSecKeyInfoCtxFinalize(&keyInfoCtx);
             xmlSecKeyDestroy(key);
             xmlFreeDoc(doc);
@@ -477,8 +565,7 @@ xmlSecSimpleKeysStoreLoad_ex(xmlSecKeyStorePtr store, const char *uri,
         if(xmlSecKeyIsValid(key)) {
             ret = adoptKeyFunc(store, key);
             if(ret < 0) {
-                xmlSecInternalError("adoptKeyFunc",
-                    xmlSecKeyStoreGetName(store));
+                xmlSecInternalError("adoptKeyFunc", xmlSecKeyStoreGetName(store));
                 xmlSecKeyDestroy(key);
                 xmlFreeDoc(doc);
                 return(-1);
@@ -534,8 +621,7 @@ xmlSecSimpleKeysStoreSave(xmlSecKeyStorePtr store, const char *filename, xmlSecK
     /* create doc */
     doc = xmlSecCreateTree(BAD_CAST "Keys", xmlSecNs);
     if(doc == NULL) {
-        xmlSecInternalError("xmlSecCreateTree",
-                            xmlSecKeyStoreGetName(store));
+        xmlSecInternalError("xmlSecCreateTree", xmlSecKeyStoreGetName(store));
         return(-1);
     }
 
@@ -550,10 +636,8 @@ xmlSecSimpleKeysStoreSave(xmlSecKeyStorePtr store, const char *filename, xmlSecK
 
         cur = xmlSecAddChild(xmlDocGetRootElement(doc), xmlSecNodeKeyInfo, xmlSecDSigNs);
         if(cur == NULL) {
-            xmlSecInternalError2("xmlSecAddChild",
-                                 xmlSecKeyStoreGetName(store),
-                                 "node=%s",
-                                 xmlSecErrorsSafeString(xmlSecNodeKeyInfo));
+            xmlSecInternalError2("xmlSecAddChild", xmlSecKeyStoreGetName(store),
+                "node=%s", xmlSecErrorsSafeString(xmlSecNodeKeyInfo));
             xmlFreeDoc(doc);
             return(-1);
         }
@@ -561,10 +645,8 @@ xmlSecSimpleKeysStoreSave(xmlSecKeyStorePtr store, const char *filename, xmlSecK
         /* special data key name */
         if(xmlSecKeyGetName(key) != NULL) {
             if(xmlSecAddChild(cur, xmlSecNodeKeyName, xmlSecDSigNs) == NULL) {
-                xmlSecInternalError2("xmlSecAddChild",
-                                     xmlSecKeyStoreGetName(store),
-                                     "node=%s",
-                                     xmlSecErrorsSafeString(xmlSecNodeKeyName));
+                xmlSecInternalError2("xmlSecAddChild", xmlSecKeyStoreGetName(store),
+                    "node=%s", xmlSecErrorsSafeString(xmlSecNodeKeyName));
                 xmlFreeDoc(doc);
                 return(-1);
             }
@@ -585,9 +667,8 @@ xmlSecSimpleKeysStoreSave(xmlSecKeyStorePtr store, const char *filename, xmlSecK
             }
 
             if(xmlSecAddChild(cur, dataId->dataNodeName, dataId->dataNodeNs) == NULL) {
-                xmlSecInternalError2("xmlSecAddChild",
-                                     xmlSecKeyStoreGetName(store),
-                                    "node=%s", xmlSecErrorsSafeString(dataId->dataNodeName));
+                xmlSecInternalError2("xmlSecAddChild", xmlSecKeyStoreGetName(store),
+                    "node=%s", xmlSecErrorsSafeString(dataId->dataNodeName));
                 xmlFreeDoc(doc);
                 return(-1);
             }
@@ -595,8 +676,7 @@ xmlSecSimpleKeysStoreSave(xmlSecKeyStorePtr store, const char *filename, xmlSecK
 
         ret = xmlSecKeyInfoCtxInitialize(&keyInfoCtx, NULL);
         if(ret < 0) {
-            xmlSecInternalError("xmlSecKeyInfoCtxInitialize",
-                                xmlSecKeyStoreGetName(store));
+            xmlSecInternalError("xmlSecKeyInfoCtxInitialize", xmlSecKeyStoreGetName(store));
             xmlFreeDoc(doc);
             return(-1);
         }
@@ -606,11 +686,19 @@ xmlSecSimpleKeysStoreSave(xmlSecKeyStorePtr store, const char *filename, xmlSecK
         keyInfoCtx.keyReq.keyType       = type;
         keyInfoCtx.keyReq.keyUsage      = xmlSecKeyDataUsageAny;
 
+        /* enable all keydata for store */
+        ret = xmlSecSimpleKeysStoreEnableAllKeyData(&keyInfoCtx);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecSimpleKeysStoreEnableAllKeyData", xmlSecKeyStoreGetName(store));
+            xmlSecKeyInfoCtxFinalize(&keyInfoCtx);
+            xmlFreeDoc(doc);
+            return(-1);
+        }
+
         /* finally write key in the node */
         ret = xmlSecKeyInfoNodeWrite(cur, key, &keyInfoCtx);
         if(ret < 0) {
-            xmlSecInternalError("xmlSecKeyInfoNodeWrite",
-                                xmlSecKeyStoreGetName(store));
+            xmlSecInternalError("xmlSecKeyInfoNodeWrite", xmlSecKeyStoreGetName(store));
             xmlSecKeyInfoCtxFinalize(&keyInfoCtx);
             xmlFreeDoc(doc);
             return(-1);
@@ -622,7 +710,7 @@ xmlSecSimpleKeysStoreSave(xmlSecKeyStorePtr store, const char *filename, xmlSecK
     ret = xmlSaveFormatFile(filename, doc, 1);
     if(ret < 0) {
         xmlSecXmlError2("xmlSaveFormatFile", xmlSecKeyStoreGetName(store),
-                        "filename=%s", xmlSecErrorsSafeString(filename));
+            "filename=%s", xmlSecErrorsSafeString(filename));
         xmlFreeDoc(doc);
         return(-1);
     }
@@ -685,8 +773,7 @@ xmlSecSimpleKeysStoreFinalize(xmlSecKeyStorePtr store) {
 }
 
 static xmlSecKeyPtr
-xmlSecSimpleKeysStoreFindKey(xmlSecKeyStorePtr store, const xmlChar* name,
-                            xmlSecKeyInfoCtxPtr keyInfoCtx) {
+xmlSecSimpleKeysStoreFindKey(xmlSecKeyStorePtr store, const xmlChar* name, xmlSecKeyInfoCtxPtr keyInfoCtx) {
     xmlSecPtrListPtr list;
     xmlSecKeyPtr key;
     xmlSecSize pos, size;
@@ -706,4 +793,3 @@ xmlSecSimpleKeysStoreFindKey(xmlSecKeyStorePtr store, const xmlChar* name,
     }
     return(NULL);
 }
-
