@@ -18,14 +18,6 @@
 
 #include <string.h>
 
-#define WIN32_NO_STATUS
-#include <windows.h>
-#undef WIN32_NO_STATUS
-#include <wincrypt.h>
-#include <ntstatus.h>
-#include <bcrypt.h>
-#include <ncrypt.h>
-
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/base64.h>
 #include <xmlsec/bn.h>
@@ -2120,15 +2112,4 @@ xmlSecMSCngKeyDataEcGetKlass(void) {
     return(&xmlSecMSCngKeyDataEcKlass);
 }
 
-/**
- * xmlSecMSCngKeyDataEcdsaGetKlass:
- *
- * Deprecated. The MSCng EC CertKey data klass.
- *
- * Returns: pointer to MSCng EC key data klass.
- */
-xmlSecKeyDataId
-xmlSecMSCngKeyDataEcdsaGetKlass(void) {
-    return(xmlSecMSCngKeyDataEcGetKlass());
-}
 #endif /* XMLSEC_NO_EC */
