@@ -21,13 +21,6 @@
 #include <string.h>
 #include <ctype.h>
 
-#define WIN32_NO_STATUS
-#include <windows.h>
-#undef WIN32_NO_STATUS
-#include <ntstatus.h>
-#include <bcrypt.h>
-#include <ncrypt.h>
-
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/base64.h>
 #include <xmlsec/keys.h>
@@ -43,7 +36,6 @@
 #include "../cast_helpers.h"
 #include "../transform_helpers.h"
 
-
 #ifndef XMLSEC_NO_EC
 
 /* Mingw has old version of bcrypt.h file */
