@@ -20,9 +20,6 @@
 
 #include <string.h>
 
-#include <windows.h>
-#include <wincrypt.h>
-
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/keys.h>
 #include <xmlsec/errors.h>
@@ -122,25 +119,6 @@ xmlSecMSCryptoAppGetCertStoreName(void) {
  * Keys
  *************************************************************************************/
 
-/**
- * xmlSecMSCryptoAppKeyLoad:
- * @filename:           the key filename.
- * @format:             the key file format.
- * @pwd:                the key file password.
- * @pwdCallback:        the key password callback.
- * @pwdCallbackCtx:     the user context for password callback.
- *
- * Deprecated, use xmlSecMSCryptoAppKeyLoadEx instead. Reads key from the a file.
- *
- * Returns: pointer to the key or NULL if an error occurs.
- */
-xmlSecKeyPtr
-xmlSecMSCryptoAppKeyLoad(const char *filename, xmlSecKeyDataFormat format,
-    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
-) {
-    return(xmlSecMSCryptoAppKeyLoadEx(filename,  xmlSecKeyDataTypeUnknown, format, pwd, pwdCallback, pwdCallbackCtx));
-}
-
 /**
  * xmlSecMSCryptoAppKeyLoadEx:
  * @filename:           the key filename.
