@@ -6,7 +6,7 @@
  * distribution for preciese wording.
  *
  * Copyright (C) 2003 Cordys R&D BV, All rights reserved.
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  * Copyright (c) 2005-2006 Cryptocom LTD (http://www.cryptocom.ru).
  */
 /**
@@ -219,7 +219,11 @@ xmlSecCryptoGetFunctions_mscrypto(void) {
 #endif /* XMLSEC_NO_SHA512 */
 
     gXmlSecMSCryptoFunctions->transformRsaPkcs1GetKlass         = xmlSecMSCryptoTransformRsaPkcs1GetKlass;
+
+#ifndef XMLSEC_NO_SHA1
     gXmlSecMSCryptoFunctions->transformRsaOaepGetKlass          = xmlSecMSCryptoTransformRsaOaepGetKlass;
+#endif /* XMLSEC_NO_SHA1 */
+
 #endif /* XMLSEC_NO_RSA */
 
     /******************************* SHA ********************************/
@@ -245,17 +249,20 @@ xmlSecCryptoGetFunctions_mscrypto(void) {
     gXmlSecMSCryptoFunctions->cryptoAppShutdown                 = xmlSecMSCryptoAppShutdown;
     gXmlSecMSCryptoFunctions->cryptoAppDefaultKeysMngrInit      = xmlSecMSCryptoAppDefaultKeysMngrInit;
     gXmlSecMSCryptoFunctions->cryptoAppDefaultKeysMngrAdoptKey  = xmlSecMSCryptoAppDefaultKeysMngrAdoptKey;
+    gXmlSecMSCryptoFunctions->cryptoAppDefaultKeysMngrVerifyKey = xmlSecMSCryptoAppDefaultKeysMngrVerifyKey;
     gXmlSecMSCryptoFunctions->cryptoAppDefaultKeysMngrLoad      = xmlSecMSCryptoAppDefaultKeysMngrLoad;
     gXmlSecMSCryptoFunctions->cryptoAppDefaultKeysMngrSave      = xmlSecMSCryptoAppDefaultKeysMngrSave;
 #ifndef XMLSEC_NO_X509
     gXmlSecMSCryptoFunctions->cryptoAppKeysMngrCertLoad         = xmlSecMSCryptoAppKeysMngrCertLoad;
     gXmlSecMSCryptoFunctions->cryptoAppKeysMngrCertLoadMemory   = xmlSecMSCryptoAppKeysMngrCertLoadMemory;
+    gXmlSecMSCryptoFunctions->cryptoAppKeysMngrCrlLoad          = xmlSecMSCryptoAppKeysMngrCrlLoad;
+    gXmlSecMSCryptoFunctions->cryptoAppKeysMngrCrlLoadMemory    = xmlSecMSCryptoAppKeysMngrCrlLoadMemory;
     gXmlSecMSCryptoFunctions->cryptoAppPkcs12Load               = xmlSecMSCryptoAppPkcs12Load;
     gXmlSecMSCryptoFunctions->cryptoAppPkcs12LoadMemory         = xmlSecMSCryptoAppPkcs12LoadMemory;
     gXmlSecMSCryptoFunctions->cryptoAppKeyCertLoad              = xmlSecMSCryptoAppKeyCertLoad;
     gXmlSecMSCryptoFunctions->cryptoAppKeyCertLoadMemory        = xmlSecMSCryptoAppKeyCertLoadMemory;
 #endif /* XMLSEC_NO_X509 */
-    gXmlSecMSCryptoFunctions->cryptoAppKeyLoad                  = xmlSecMSCryptoAppKeyLoad;
+    gXmlSecMSCryptoFunctions->cryptoAppKeyLoadEx                = xmlSecMSCryptoAppKeyLoadEx;
     gXmlSecMSCryptoFunctions->cryptoAppKeyLoadMemory            = xmlSecMSCryptoAppKeyLoadMemory;
     gXmlSecMSCryptoFunctions->cryptoAppDefaultPwdCallback       = (void*)xmlSecMSCryptoAppGetDefaultPwdCallback();
 
@@ -349,8 +356,7 @@ static xmlSecMSCryptoProviderInfo xmlSecMSCryptoProviderInfo_Random[] = {
  * @buffer:             the destination buffer.
  * @size:               the numer of bytes to generate.
  *
- * Generates @size random bytes and puts result in @buffer
- * (not implemented yet).
+ * Generates @size random bytes and puts result in @buffer.
  *
  * Returns: 0 on success or a negative value otherwise.
  */
@@ -471,97 +477,6 @@ xmlSecMSCryptoErrorsDefaultCallback(const char* file, int line, const char* func
     xmlSecErrorsDefaultCallback(file, line, func, errorObject, errorSubject, reason, msg);
 }
 
-/**
- * xmlSecMSCryptoConvertUtf8ToUnicode:
- * @str:         the string to convert.
- *
- * Converts input string from UTF8 to Unicode.
- *
- * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
- */
-LPWSTR
-xmlSecMSCryptoConvertUtf8ToUnicode(const xmlChar* str) {
-    return(xmlSecWin32ConvertUtf8ToUnicode(str));
-}
-
-/**
- * xmlSecMSCryptoConvertUnicodeToUtf8:
- * @str:         the string to convert.
- *
- * Converts input string from Unicode to UTF8.
- *
- * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
- */
-xmlChar*
-xmlSecMSCryptoConvertUnicodeToUtf8(LPCWSTR str) {
-    return(xmlSecWin32ConvertUnicodeToUtf8(str));
-}
-
-/**
- * xmlSecMSCryptoConvertLocaleToUnicode:
- * @str:         the string to convert.
- *
- * Converts input string from current system locale to Unicode.
- *
- * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
- */
-LPWSTR
-xmlSecMSCryptoConvertLocaleToUnicode(const char* str) {
-    return(xmlSecWin32ConvertLocaleToUnicode(str));
-}
-
-/**
- * xmlSecMSCryptoConvertLocaleToUtf8:
- * @str:         the string to convert.
- *
- * Converts input string from locale to UTF8.
- *
- * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
- */
-xmlChar*
-xmlSecMSCryptoConvertLocaleToUtf8(const char * str) {
-    return(xmlSecWin32ConvertLocaleToUtf8(str));
-}
-
-/**
- * xmlSecMSCryptoConvertUtf8ToLocale:
- * @str:         the string to convert.
- *
- * Converts input string from UTF8 to locale.
- *
- * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
- */
-char *
-xmlSecMSCryptoConvertUtf8ToLocale(const xmlChar* str) {
-    return(xmlSecWin32ConvertUtf8ToLocale(str));
-}
-
-/**
- * xmlSecMSCryptoConvertTstrToUtf8:
- * @str:         the string to convert.
- *
- * Converts input string from TSTR (locale or Unicode) to UTF8.
- *
- * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
- */
-xmlChar*
-xmlSecMSCryptoConvertTstrToUtf8(LPCTSTR str) {
-    return(xmlSecWin32ConvertTstrToUtf8(str));
-}
-
-/**
- * xmlSecMSCryptoConvertUtf8ToTstr:
- * @str:         the string to convert.
- *
- * Converts input string from UTF8 to TSTR (locale or Unicode).
- *
- * Returns: a pointer to newly allocated string (must be freed with xmlFree) or NULL if an error occurs.
- */
-LPTSTR
-xmlSecMSCryptoConvertUtf8ToTstr(const xmlChar*  str) {
-    return(xmlSecWin32ConvertUtf8ToTstr(str));
-}
-
 /********************************************************************
  *
  * Crypto Providers
@@ -689,5 +604,3 @@ ConvertEndianInPlace(xmlSecByte * buf, xmlSecSize size) {
     }
     return (0);
 }
-
-
