@@ -19,8 +19,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <windows.h>
-#include <wincrypt.h>
 
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/base64.h>
