@@ -17,9 +17,8 @@
 #include "globals.h"
 
 #include <string.h>
+#include <stdlib.h>
 
-#include <windows.h>
-#include <wincrypt.h>
 #ifndef XMLSEC_NO_GOST
 #include "csp_calg.h"
 #endif
