@@ -223,7 +223,7 @@ xmlSecNssAppAscii2UCS2Conv(PRBool toUnicode,
                            unsigned int  *outBufLen,
                            PRBool         swapBytes ATTRIBUTE_UNUSED)
 {
-    SECItem it;
+    SECItem it = { siBuffer, NULL, 0 };
 
     if (toUnicode == PR_FALSE) {
         return (PR_FALSE);
@@ -272,25 +272,6 @@ xmlSecNssAppNicknameCollisionCallback(SECItem *old_nick ATTRIBUTE_UNUSED,
 }
 #endif /* XMLSEC_NO_X509 */
 
-/**
- * xmlSecNssAppKeyLoad:
- * @filename:           the key filename.
- * @format:             the key file format.
- * @pwd:                the key file password.
- * @pwdCallback:        the key password callback.
- * @pwdCallbackCtx:     the user context for password callback.
- *
- * Deprecated, use @xmlSecNssAppKeyLoadEx instead. Reads key from a file
- *
- * Returns: pointer to the key or NULL if an error occurs.
- */
-xmlSecKeyPtr
-xmlSecNssAppKeyLoad(const char *filename, xmlSecKeyDataFormat format,
-    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
-) {
-    return(xmlSecNssAppKeyLoadEx(filename, xmlSecKeyDataTypeUnknown, format, pwd, pwdCallback, pwdCallbackCtx));
-}
-
 /**
  * xmlSecNssAppKeyLoadEx:
  * @filename:           the key filename.
@@ -308,7 +289,7 @@ xmlSecKeyPtr
 xmlSecNssAppKeyLoadEx(const char *filename, xmlSecKeyDataType type ATTRIBUTE_UNUSED, xmlSecKeyDataFormat format,
     const char *pwd, void* pwdCallback, void* pwdCallbackCtx
 ) {
-    SECItem secItem;
+    SECItem secItem = { siBuffer, NULL, 0 };
     xmlSecKeyPtr res;
     int ret;
 
@@ -351,7 +332,7 @@ xmlSecNssAppKeyLoadEx(const char *filename, xmlSecKeyDataType type ATTRIBUTE_UNU
 xmlSecKeyPtr
 xmlSecNssAppKeyLoadMemory(const xmlSecByte* data, xmlSecSize dataSize, xmlSecKeyDataFormat format,
                     const char *pwd, void* pwdCallback, void* pwdCallbackCtx) {
-    SECItem secItem;
+    SECItem secItem = { siBuffer, NULL, 0 };
     xmlSecKeyPtr res;
     int ret;
 
@@ -407,6 +388,7 @@ xmlSecNssAppKeyLoadSECItem(SECItem* secItem, xmlSecKeyDataFormat format,
         }
         break;
     case xmlSecKeyDataFormatCertDer:
+    case xmlSecKeyDataFormatCertPem:
         key = xmlSecNssAppKeyFromCertLoadSECItem(secItem, format);
         if(key == NULL) {
             xmlSecInternalError("xmlSecNssAppKeyFromCertLoadSECItem", NULL);
@@ -439,7 +421,7 @@ xmlSecNssAppDerKeyLoadSECItem(SECItem* secItem) {
     SECKEYPublicKey *pubkey = NULL;
     SECKEYPrivateKey *privkey = NULL;
     CERTSubjectPublicKeyInfo *spki = NULL;
-    SECItem nickname;
+    SECItem nickname = { siBuffer, NULL, 0 };
     PK11SlotInfo *slot = NULL;
     SECStatus status;
 
@@ -528,6 +510,73 @@ done:
     return (retval);
 }
 
+/* returns 1 if matches, 0 if not, or a negative value on error */
+static int
+xmlSecNssAppCheckCertMatchesKey(xmlSecKeyPtr key,  CERTCertificate * cert) {
+    xmlSecKeyDataPtr keyData = NULL;
+    SECKEYPublicKey* pubkey = NULL;
+    SECKEYPublicKey* cert_pubkey = NULL;
+    SECItem * der_pubkey = NULL;
+    SECItem * der_cert_pubkey = NULL;
+    int res = -1;
+
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(cert != NULL, -1);
+
+    /* get key's pubkey and its der encoding */
+    keyData = xmlSecKeyGetValue(key);
+    if(keyData == NULL) {
+        res = 0; /* no key -> no match */
+        goto done;
+    }
+    pubkey = xmlSecNssPKIKeyDataGetPubKey(keyData);
+    if(pubkey == NULL) {
+        xmlSecInternalError("xmlSecNssPKIKeyDataGetPubKey", NULL);
+        goto done;
+    }
+    der_pubkey = SECKEY_EncodeDERSubjectPublicKeyInfo(pubkey);
+    if (der_pubkey == NULL) {
+        xmlSecNssError("SECKEY_EncodeDERSubjectPublicKeyInfo", NULL);
+        goto done;
+    }
+
+    /* get certs's pubkey and its der encoding */
+    cert_pubkey = CERT_ExtractPublicKey(cert);
+    if (cert_pubkey == NULL) {
+        xmlSecNssError("CERT_ExtractPublicKey", NULL);
+        goto done;
+    }
+    der_cert_pubkey = SECKEY_EncodeDERSubjectPublicKeyInfo(cert_pubkey);
+    if (der_cert_pubkey == NULL) {
+        xmlSecNssError("SECKEY_EncodeDERSubjectPublicKeyInfo", NULL);
+        goto done;
+    }
+
+    /* compare */
+    if(SECEqual == SECITEM_CompareItem(der_pubkey, der_cert_pubkey)) {
+        /* match */
+        res = 1;
+    } else {
+        /* no match */
+        res = 0;
+    }
+
+done:
+    if(pubkey != NULL) {
+        SECKEY_DestroyPublicKey(pubkey);
+    }
+    if (cert_pubkey) {
+        SECKEY_DestroyPublicKey(cert_pubkey);
+    }
+    if(der_pubkey != NULL) {
+        SECITEM_FreeItem(der_pubkey, PR_TRUE);
+    }
+    if(der_cert_pubkey != NULL) {
+        SECITEM_FreeItem(der_cert_pubkey, PR_TRUE);
+    }
+    return(res);
+}
+
 #ifndef XMLSEC_NO_X509
 /**
  * xmlSecNssAppKeyCertLoad:
@@ -541,7 +590,7 @@ done:
  */
 int
 xmlSecNssAppKeyCertLoad(xmlSecKeyPtr key, const char* filename, xmlSecKeyDataFormat format) {
-    SECItem secItem;
+    SECItem secItem = { siBuffer, NULL, 0 };
     int ret;
 
     xmlSecAssert2(key != NULL, -1);
@@ -580,7 +629,7 @@ xmlSecNssAppKeyCertLoad(xmlSecKeyPtr key, const char* filename, xmlSecKeyDataFor
  */
 int
 xmlSecNssAppKeyCertLoadMemory(xmlSecKeyPtr key, const xmlSecByte* data, xmlSecSize dataSize, xmlSecKeyDataFormat format) {
-    SECItem secItem;
+    SECItem secItem = { siBuffer, NULL, 0 };
     int ret;
 
     xmlSecAssert2(key != NULL, -1);
@@ -620,6 +669,7 @@ int
 xmlSecNssAppKeyCertLoadSECItem(xmlSecKeyPtr key, SECItem* secItem, xmlSecKeyDataFormat format) {
     CERTCertificate *cert = NULL;
     xmlSecKeyDataPtr x509Data;
+    int isKeyCert = 0;
     int ret;
     int res = -1;
 
@@ -641,6 +691,14 @@ xmlSecNssAppKeyCertLoadSECItem(xmlSecKeyPtr key, SECItem* secItem, xmlSecKeyData
             goto done;
         }
         break;
+    case xmlSecKeyDataFormatCertPem:
+        cert = xmlSecNssX509CertPemRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
+        if(cert == NULL) {
+            xmlSecInternalError2("xmlSecNssX509CertPemRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            goto done;
+        }
+        break;
     default:
         xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
             "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
@@ -654,8 +712,19 @@ xmlSecNssAppKeyCertLoadSECItem(xmlSecKeyPtr key, SECItem* secItem, xmlSecKeyData
         xmlSecInternalError("xmlSecKeyEnsureData(xmlSecNssKeyDataX509Id)", NULL);
         goto done;
     }
+
+    /* do we want to add this cert as a key cert? */
     if(xmlSecNssKeyDataX509GetKeyCert(x509Data) == NULL) {
-        /* TODO: check if cert matches the key */
+        ret = xmlSecNssAppCheckCertMatchesKey(key, cert);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecNssAppCheckCertMatchesKey", NULL);
+            goto done;
+        }
+        if(ret == 1) {
+            isKeyCert = 1;
+        }
+    }
+    if(isKeyCert != 0) {
         ret = xmlSecNssKeyDataX509AdoptKeyCert(x509Data, cert);
         if(ret < 0) {
             xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert", NULL);
@@ -697,7 +766,7 @@ xmlSecKeyPtr
 xmlSecNssAppPkcs12Load(const char *filename, const char *pwd,
                        void *pwdCallback ATTRIBUTE_UNUSED,
                        void* pwdCallbackCtx ATTRIBUTE_UNUSED) {
-    SECItem secItem;
+    SECItem secItem = { siBuffer, NULL, 0 };
     xmlSecKeyPtr res;
     int ret;
 
@@ -740,7 +809,7 @@ xmlSecKeyPtr
 xmlSecNssAppPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize, const char *pwd,
                        void *pwdCallback ATTRIBUTE_UNUSED,
                        void* pwdCallbackCtx ATTRIBUTE_UNUSED) {
-    SECItem secItem;
+    SECItem secItem = { siBuffer, NULL, 0 };
     xmlSecKeyPtr res;
     int ret;
 
@@ -787,15 +856,15 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
     xmlSecKeyDataPtr x509Data = NULL;
     int ret;
     PK11SlotInfo *slot = NULL;
-    SECItem pwditem;
-    SECItem uc2_pwditem;
+    SECItem pwditem = { siBuffer, NULL, 0 };
+    SECItem uc2_pwditem = { siBuffer, NULL, 0 };
     SECStatus rv;
     SECKEYPrivateKey *privkey = NULL;
     SECKEYPublicKey *pubkey = NULL;
     CERTCertList *certlist = NULL;
-    CERTCertListNode    *head = NULL;
-    CERTCertificate     *cert = NULL;
-    CERTCertificate     *tmpcert = NULL;
+    CERTCertListNode *head = NULL;
+    CERTCertificate *cert = NULL;
+    CERTCertificate *tmpcert = NULL;
     SEC_PKCS12DecoderContext *p12ctx = NULL;
     const SEC_PKCS12DecoderItem *dip;
     size_t pwdSize;
@@ -913,7 +982,6 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
             ret = xmlSecNssKeyDataX509AdoptKeyCert(x509Data, tmpcert);
             if(ret < 0) {
                 xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert", NULL);
-                CERT_DestroyCertificate(tmpcert);
                 goto done;
             }
             tmpcert = NULL; /* owned by x509Data now */
@@ -926,7 +994,6 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
             ret = xmlSecNssKeyDataX509AdoptCert(x509Data, tmpcert);
             if(ret < 0) {
                 xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert", NULL);
-                CERT_DestroyCertificate(tmpcert);
                 goto done;
             }
             tmpcert = NULL; /* owned by x509Data now */
@@ -984,6 +1051,9 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
     key = NULL;
 
 done:
+    if(tmpcert != NULL) {
+        CERT_DestroyCertificate(tmpcert);
+    }
     if(key != NULL) {
         xmlSecKeyDestroy(key);
     }
@@ -1047,6 +1117,14 @@ xmlSecNssAppKeyFromCertLoadSECItem(SECItem* secItem, xmlSecKeyDataFormat format)
             goto done;
         }
         break;
+    case xmlSecKeyDataFormatCertPem:
+        cert = xmlSecNssX509CertPemRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
+        if(cert == NULL) {
+            xmlSecInternalError2("xmlSecNssX509CertPemRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            goto done;
+        }
+        break;
     default:
         xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
             "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
@@ -1119,7 +1197,7 @@ int
 xmlSecNssAppKeysMngrCertLoad(xmlSecKeysMngrPtr mngr, const char *filename,
                              xmlSecKeyDataFormat format,
                              xmlSecKeyDataType type) {
-    SECItem secItem;
+    SECItem secItem = { siBuffer, NULL, 0 };
     int ret;
 
     xmlSecAssert2(mngr != NULL, -1);
@@ -1162,7 +1240,7 @@ int
 xmlSecNssAppKeysMngrCertLoadMemory(xmlSecKeysMngrPtr mngr, const xmlSecByte* data,
                              xmlSecSize dataSize, xmlSecKeyDataFormat format,
                              xmlSecKeyDataType type) {
-    SECItem secItem;
+    SECItem secItem = { siBuffer, NULL, 0 };
     int ret;
 
     xmlSecAssert2(mngr != NULL, -1);
@@ -1229,6 +1307,14 @@ xmlSecNssAppKeysMngrCertLoadSECItem(xmlSecKeysMngrPtr mngr, SECItem* secItem,
             return(-1);
         }
         break;
+    case xmlSecKeyDataFormatCertPem:
+        cert = xmlSecNssX509CertPemRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
+        if(cert == NULL) {
+            xmlSecInternalError2("xmlSecNssX509CertPemRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            return(-1);
+        }
+        break;
     default:
         xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
             "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
@@ -1259,7 +1345,7 @@ int
 xmlSecNssAppKeysMngrCrlLoad(xmlSecKeysMngrPtr mngr, const char *filename, xmlSecKeyDataFormat format) {
     xmlSecKeyDataStorePtr x509Store;
     CERTSignedCrl* crl;
-    SECItem secItem;
+    SECItem secItem = { siBuffer, NULL, 0 };
     int ret;
 
     xmlSecAssert2(mngr != NULL, -1);
@@ -1327,7 +1413,7 @@ int
 xmlSecNssAppKeysMngrCrlLoadMemory(xmlSecKeysMngrPtr mngr, const xmlSecByte* data, xmlSecSize dataSize, xmlSecKeyDataFormat format) {
     xmlSecKeyDataStorePtr x509Store;
     CERTSignedCrl* crl;
-    SECItem secItem;
+    SECItem secItem = { siBuffer, NULL, 0 };
     int ret;
 
     xmlSecAssert2(mngr != NULL, -1);
@@ -1476,13 +1562,31 @@ xmlSecNssAppDefaultKeysMngrAdoptKey(xmlSecKeysMngrPtr mngr, xmlSecKeyPtr key) {
  */
 int
 xmlSecNssAppDefaultKeysMngrVerifyKey(xmlSecKeysMngrPtr mngr, xmlSecKeyPtr key, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+#ifndef XMLSEC_NO_X509
+    xmlSecKeyDataStorePtr x509Store;
 
     xmlSecAssert2(mngr != NULL, -1);
     xmlSecAssert2(key != NULL, -1);
     xmlSecAssert2(keyInfoCtx != NULL, -1);
 
-    xmlSecNotImplementedError("xmlSecNssAppDefaultKeysMngrVerifyKey");
+    x509Store = xmlSecKeysMngrGetDataStore(mngr, xmlSecNssX509StoreId);
+    if(x509Store == NULL) {
+        xmlSecInternalError("xmlSecKeysMngrGetDataStore(xmlSecNssX509StoreId)", NULL);
+        return(-1);
+    }
+
+    return(xmlSecNssX509StoreVerifyKey(x509Store, key, keyInfoCtx));
+
+#else  /* XMLSEC_NO_X509 */
+
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+
+    xmlSecNotImplementedError("X509 support is disabled");
     return(-1);
+
+#endif /* XMLSEC_NO_X509 */
 }
 
 /**
