@@ -40,6 +40,11 @@
 #define XMLSEC_NSS_GCM_CIPHER_IV_SIZE                    12
 #define XMLSEC_NSS_GCM_CIPHER_TAG_SIZE                   16
 
+/* struct was renamed in https://github.com/nss-dev/nss/commit/ba931199b924a2eac38899d04b04eedc75771546 */
+#if (NSS_VMAJOR < 3) || ((NSS_VMAJOR == 3) && (NSS_VMINOR < 52))
+#define CK_NSS_GCM_PARAMS CK_GCM_PARAMS
+#endif // (NSS_VMAJOR < 3) || ((NSS_VMAJOR == 3) && (NSS_VMINOR < 52))
+
 /**************************************************************************
  *
  * Internal Nss GCM cipher CTX
@@ -200,7 +205,7 @@ xmlSecNssGcmCipherGetKey(xmlSecNssGcmCipherCtxPtr ctx, CK_ATTRIBUTE_TYPE operati
     xmlSecBufferPtr keyBuf;
     xmlSecSize keySize;
     xmlSecByte* keyData;
-    SECItem key_item;
+    SECItem key_item = { siBuffer, NULL, 0 };
     PK11SlotInfo *slot = NULL;
     PK11SymKey *symKey = NULL;
 
@@ -246,7 +251,7 @@ xmlSecNssGcmCipherEncrypt(xmlSecNssGcmCipherCtxPtr ctx, xmlSecBufferPtr in, xmlS
     xmlSecSize inSize, outSize;
     xmlSecByte *plaintext, *iv, *outData;
     CK_NSS_GCM_PARAMS gcm_params;
-    SECItem param;
+    SECItem param = { siBuffer, NULL, 0 };
     PK11SymKey* symKey = NULL;
     unsigned int outputlen = 0, maxoutputlen, inputlen;
     SECStatus rv;
@@ -334,7 +339,7 @@ xmlSecNssGcmCipherDecrypt(xmlSecNssGcmCipherCtxPtr ctx, xmlSecBufferPtr in, xmlS
     xmlSecSize inSize, outSize;
     xmlSecByte *iv, *ciphertext, *outData;
     CK_NSS_GCM_PARAMS gcm_params;
-    SECItem param;
+    SECItem param = { siBuffer, NULL, 0 };
     PK11SymKey* symKey = NULL;
     unsigned int outputlen = 0, maxoutputlen, inputlen;
     SECStatus rv;
