@@ -1,18 +1,16 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * HMAC transforms implementation for NSS.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  * Copyright (c) 2003 America Online, Inc.  All rights reserved.
  */
 /**
- * SECTION:hmac
- * @Short_description: HMAC transforms implementation for NSS.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #ifndef XMLSEC_NO_HMAC
@@ -23,7 +21,7 @@
 #include <nspr.h>
 #include <nss.h>
 #include <secoid.h>
-#include <pk11pub.h>
+#include <pk11func.h>
 
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/keys.h>
@@ -44,37 +42,6 @@
  *
  *****************************************************************************/
 
-/**
- * xmlSecNssHmacGetMinOutputLength:
- *
- * DEPRECATED (use @xmlSecTransformHmacGetMinOutputBitsSize instead).
- * Gets the value of min HMAC length.
- *
- * Returns: the min HMAC output length
- */
-int xmlSecNssHmacGetMinOutputLength(void)
-{
-    xmlSecSize val = xmlSecTransformHmacGetMinOutputBitsSize();
-    int res;
-
-    XMLSEC_SAFE_CAST_SIZE_TO_INT(val, res, return(-1), NULL);
-    return res;
-}
-
-/**
- * xmlSecNssHmacSetMinOutputLength:
- * @min_length: the new min length
- *
- * DEPRECATED (use @xmlSecTransformHmacSetMinOutputBitsSize instead).
- * Sets the min HMAC output length
- */
-void xmlSecNssHmacSetMinOutputLength(int min_length)
-{
-    xmlSecSize val;
-    XMLSEC_SAFE_CAST_INT_TO_SIZE(min_length, val, return, NULL);
-    xmlSecTransformHmacSetMinOutputBitsSize(val);
-}
-
 /**************************************************************************
  *
  * Internal NSS HMAC CTX
@@ -85,7 +52,7 @@ struct _xmlSecNssHmacCtx {
     CK_MECHANISM_TYPE   digestType;
     PK11Context*        digestCtx;
     xmlSecByte          dgst[XMLSEC_TRASNFORM_HMAC_MAX_OUTPUT_SIZE];
-    xmlSecSize          dgstSize;       /* dgst size in bits */
+    xmlSecSize          dgstSizeInBits;       /* dgst size in bits */
 };
 
 /******************************************************************************
@@ -243,9 +210,6 @@ xmlSecNssHmacFinalize(xmlSecTransformPtr transform) {
     memset(ctx, 0, sizeof(xmlSecNssHmacCtx));
 }
 
-/**
- * xmlSecNssHmacNodeRead:
- */
 static int
 xmlSecNssHmacNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
                       xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
@@ -260,12 +224,13 @@ xmlSecNssHmacNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
     ctx = xmlSecNssHmacGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
 
-    ret = xmlSecTransformHmacReadOutputBitsSize(node, ctx->dgstSize, &ctx->dgstSize);
+    ret = xmlSecTransformHmacReadOutputBitsSize(node, ctx->dgstSizeInBits, &ctx->dgstSizeInBits);
     if (ret < 0) {
         xmlSecInternalError("xmlSecTransformHmacReadOutputBitsSize()",
             xmlSecTransformGetName(transform));
         return(-1);
     }
+    xmlSecAssert2(XMLSEC_TRASNFORM_HMAC_BITS_TO_BYTES(ctx->dgstSizeInBits) <= XMLSEC_TRASNFORM_HMAC_MAX_OUTPUT_SIZE, -1);
 
     return(0);
 }
@@ -300,8 +265,8 @@ xmlSecNssHmacSetKey(xmlSecTransformPtr transform, xmlSecKeyPtr key) {
     xmlSecKeyDataPtr value;
     xmlSecBufferPtr buffer;
     xmlSecSize bufferSize;
-    SECItem keyItem;
-    SECItem ignore;
+    SECItem keyItem = { siBuffer, NULL, 0 };
+    SECItem ignore = { siBuffer, NULL, 0 };
     PK11SlotInfo* slot;
     PK11SymKey* symKey;
 
@@ -363,11 +328,8 @@ static int
 xmlSecNssHmacVerify(xmlSecTransformPtr transform,
                         const xmlSecByte* data, xmlSecSize dataSize,
                         xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
-    static xmlSecByte last_byte_masks[] =
-                { 0xFF, 0x80, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE };
-
     xmlSecNssHmacCtxPtr ctx;
-    xmlSecByte mask;
+    int ret;
 
     xmlSecAssert2(xmlSecTransformIsValid(transform), -1);
     xmlSecAssert2(transform->operation == xmlSecTransformOperationVerify, -1);
@@ -379,38 +341,21 @@ xmlSecNssHmacVerify(xmlSecTransformPtr transform,
     ctx = xmlSecNssHmacGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
     xmlSecAssert2(ctx->digestCtx != NULL, -1);
-    xmlSecAssert2(ctx->dgstSize > 0, -1);
+    xmlSecAssert2(ctx->dgstSizeInBits > 0, -1);
 
-    /* compare the digest size in bytes */
-    if(dataSize != ((ctx->dgstSize + 7) / 8)){
-        xmlSecInvalidSizeError("HMAC digest",
-                               dataSize, ((ctx->dgstSize + 7) / 8),
-                               xmlSecTransformGetName(transform));
+    /* Returns 1 for match, 0 for no match, <0 for errors. */
+    ret = xmlSecTransformHmacVerify(data, dataSize, ctx->dgst, ctx->dgstSizeInBits, sizeof(ctx->dgst));
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecTransformHmacVerify", xmlSecTransformGetName(transform));
+        return(-1);
+    }
+    if(ret == 1) {
+        transform->status = xmlSecTransformStatusOk;
+    } else {
         transform->status = xmlSecTransformStatusFail;
-        return(0);
     }
 
-    /* we check the last byte separately */
-    xmlSecAssert2(dataSize > 0, -1);
-    mask = last_byte_masks[ctx->dgstSize % 8];
-    if((ctx->dgst[dataSize - 1] & mask) != (data[dataSize - 1]  & mask)) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_DATA_NOT_MATCH,
-                         xmlSecTransformGetName(transform),
-                         "data and digest do not match (last byte)");
-        transform->status = xmlSecTransformStatusFail;
-        return(0);
-    }
-
-    /* now check the rest of the digest */
-    if((dataSize > 1) && (memcmp(ctx->dgst, data, dataSize - 1) != 0)) {
-        xmlSecOtherError(XMLSEC_ERRORS_R_DATA_NOT_MATCH,
-                         xmlSecTransformGetName(transform),
-                         "data and digest do not match");
-        transform->status = xmlSecTransformStatusFail;
-        return(0);
-    }
-
-    transform->status = xmlSecTransformStatusOk;
+    /* done */
     return(0);
 }
 
@@ -477,23 +422,15 @@ xmlSecNssHmacExecute(xmlSecTransformPtr transform, int last, xmlSecTransformCtxP
             XMLSEC_SAFE_CAST_UINT_TO_SIZE(dgstLen, dgstSize, return(-1), xmlSecTransformGetName(transform));
 
             /* check/set the result digest size */
-            if(ctx->dgstSize == 0) {
-                ctx->dgstSize = dgstSize * 8; /* no dgst size specified, use all we have */
-            } else if(ctx->dgstSize <= 8 * dgstSize) {
-                xmlSecSize adjustedDigestSize = ((ctx->dgstSize + 7) / 8); /* we need to truncate result digest */
-                XMLSEC_SAFE_CAST_SIZE_TO_UINT(adjustedDigestSize, dgstSize, return(-1), xmlSecTransformGetName(transform));
-            } else {
-                xmlSecInvalidSizeLessThanError("HMAC digest (bits)",
-                                        8 * dgstSize, ctx->dgstSize,
-                                        xmlSecTransformGetName(transform));
-                return(-1);
+            if(ctx->dgstSizeInBits == 0) {
+                ctx->dgstSizeInBits = dgstSize * 8; /* no dgst size specified, use all we have */
             }
 
+            /* write results if needed */
             if(transform->operation == xmlSecTransformOperationSign) {
-                ret = xmlSecBufferAppend(out, ctx->dgst, dgstSize);
+                ret = xmlSecTransformHmacWriteOutput(ctx->dgst, ctx->dgstSizeInBits, dgstSize, out);
                 if(ret < 0) {
-                    xmlSecInternalError2("xmlSecBufferAppend", xmlSecTransformGetName(transform),
-                        "size=" XMLSEC_SIZE_FMT, dgstSize);
+                    xmlSecInternalError("xmlSecTransformHmacWriteOutput", xmlSecTransformGetName(transform));
                     return(-1);
                 }
             }
@@ -834,5 +771,3 @@ xmlSecNssTransformHmacSha512GetKlass(void) {
 #endif /* XMLSEC_NO_SHA512 */
 
 #endif /* XMLSEC_NO_HMAC */
-
-
