@@ -1,18 +1,16 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * DES Key Transport transforms implementation for NSS.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
  * Copyright (c) 2003 America Online, Inc.  All rights reserved.
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:kw_des
- * @Short_description: DES Key Transport transforms implementation for NSS.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #ifndef XMLSEC_NO_DES
@@ -23,7 +21,7 @@
 #include <string.h>
 
 #include <nss.h>
-#include <pk11pub.h>
+#include <pk11func.h>
 #include <hasht.h>
 
 #include <xmlsec/xmlsec.h>
@@ -233,7 +231,7 @@ xmlSecNssKWDes3SetKey(xmlSecTransformPtr transform, xmlSecKeyPtr key) {
 
 static int
 xmlSecNssKWDes3Execute(xmlSecTransformPtr transform, int last,
-                       xmlSecTransformCtxPtr transformCtx XMLSEC_ATTRIBUTE_UNUSED) {
+                       xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
     xmlSecNssKWDes3CtxPtr ctx;
     int ret;
 
@@ -257,7 +255,7 @@ xmlSecNssKWDes3Execute(xmlSecTransformPtr transform, int last,
  *
  *********************************************************************/
 static int
-xmlSecNssKWDes3Sha1(xmlSecTransformPtr transform XMLSEC_ATTRIBUTE_UNUSED,
+xmlSecNssKWDes3Sha1(xmlSecTransformPtr transform ATTRIBUTE_UNUSED,
                     const xmlSecByte * in, xmlSecSize inSize,
                     xmlSecByte * out, xmlSecSize outSize,
                     xmlSecSize * outWritten) {
@@ -312,7 +310,7 @@ xmlSecNssKWDes3Sha1(xmlSecTransformPtr transform XMLSEC_ATTRIBUTE_UNUSED,
 }
 
 static int
-xmlSecNssKWDes3GenerateRandom(xmlSecTransformPtr transform XMLSEC_ATTRIBUTE_UNUSED,
+xmlSecNssKWDes3GenerateRandom(xmlSecTransformPtr transform ATTRIBUTE_UNUSED,
                               xmlSecByte * out, xmlSecSize outSize,
                               xmlSecSize * outWritten) {
     SECStatus status;
@@ -420,7 +418,8 @@ xmlSecNssKWDes3Encrypt(const xmlSecByte *key, xmlSecSize keySize,
     PK11SymKey* symKey = NULL;
     SECItem* param = NULL;
     PK11Context* pk11ctx = NULL;
-    SECItem keyItem, ivItem;
+    SECItem keyItem = { siBuffer, NULL, 0 };
+    SECItem ivItem = { siBuffer, NULL, 0 };
     SECStatus status;
     int inLen, outLen, maxOutLen;
     int res = -1;
@@ -504,5 +503,9 @@ done:
 }
 
 
-#endif /* XMLSEC_NO_DES */
+#else /* XMLSEC_NO_DES */
 
+/* ISO C forbids an empty translation unit */
+typedef int make_iso_compilers_happy;
+
+#endif /* XMLSEC_NO_DES */
