@@ -1,17 +1,15 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * Ciphers transforms implementation for OpenSSL.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:ciphers
- * @Short_description: Ciphers transforms implementation for OpenSSL.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #include "globals.h"
@@ -24,8 +22,8 @@
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/errors.h>
 #include <xmlsec/keys.h>
-#include <xmlsec/transforms.h>
 #include <xmlsec/private.h>
+#include <xmlsec/transforms.h>
 
 #include <xmlsec/openssl/crypto.h>
 #include <xmlsec/openssl/evp.h>
@@ -250,6 +248,7 @@ xmlSecOpenSSLEvpBlockCipherCtxUpdateBlock(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
         int outLen2 = 0;
 
         if(ctx->cbcMode == 0) {
+            xmlSecAssert2(tagData != NULL, -1);
             if(!EVP_CIPHER_CTX_encrypting(ctx->cipherCtx)) {
                 ret = EVP_CIPHER_CTX_ctrl(ctx->cipherCtx, EVP_CTRL_GCM_SET_TAG,
                     XMLSEC_OPENSSL_AES_GCM_TAG_SIZE, tagData);
@@ -267,6 +266,7 @@ xmlSecOpenSSLEvpBlockCipherCtxUpdateBlock(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
         }
 
         if(ctx->cbcMode == 0) {
+            xmlSecAssert2(tagData != NULL, -1);
             if(EVP_CIPHER_CTX_encrypting(ctx->cipherCtx)) {
                 ret = EVP_CIPHER_CTX_ctrl(ctx->cipherCtx, EVP_CTRL_GCM_GET_TAG,
                     XMLSEC_OPENSSL_AES_GCM_TAG_SIZE, tagData);
@@ -388,7 +388,7 @@ xmlSecOpenSSLEvpBlockCipherCBCCtxFinal(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
         xmlSecBufferPtr in,
         xmlSecBufferPtr out,
         const xmlChar* cipherName,
-        xmlSecTransformCtxPtr transformCtx XMLSEC_ATTRIBUTE_UNUSED)
+        xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED)
 {
     xmlSecSize size, inSize, outSize;
     int inLen, outLen, padLen, blockLen;
@@ -606,12 +606,7 @@ xmlSecOpenSSLEvpBlockCipherCtxFinal(xmlSecOpenSSLEvpBlockCipherCtxPtr ctx,
     if (ctx->cbcMode) {
         return xmlSecOpenSSLEvpBlockCipherCBCCtxFinal(ctx, in, out, cipherName, transformCtx);
     } else {
-#ifndef XMLSEC_NO_AES
         return xmlSecOpenSSLEvpBlockCipherGCMCtxFinal(ctx, in, out, cipherName, transformCtx);
-#else /* XMLSEC_NO_AES */
-        xmlSecNotImplementedError("AES-GCM support is disabled");
-        return(-1);
-#endif /* XMLSEC_NO_AES */
     }
 }
 
@@ -1200,4 +1195,3 @@ xmlSecOpenSSLTransformDes3CbcGetKlass(void) {
     return(&xmlSecOpenSSLDes3CbcKlass);
 }
 #endif /* XMLSEC_NO_DES */
-
