@@ -1,17 +1,15 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * RSA Key Transport transforms implementation for OpenSSL.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
- * SECTION:kt_rsa
- * @Short_description: RSA Key Transport transforms implementation for OpenSSL.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #include "globals.h"
@@ -36,7 +34,6 @@
 
 #include <xmlsec/openssl/crypto.h>
 #include <xmlsec/openssl/evp.h>
-#include <xmlsec/openssl/bn.h>
 #include "openssl_compat.h"
 
 #ifdef XMLSEC_OPENSSL_API_300
@@ -47,41 +44,6 @@
 #include "../cast_helpers.h"
 #include "../transform_helpers.h"
 
-#ifdef OPENSSL_IS_BORINGSSL
-
-/* defined in boringssl/crypto/fipsmodule/rsa/internal.h */
-int RSA_padding_check_PKCS1_OAEP_mgf1(uint8_t *out, size_t *out_len, size_t max_out,
-                                      const uint8_t *from, size_t from_len,
-                                      const uint8_t *param, size_t param_len,
-                                      const EVP_MD *md, const EVP_MD *mgf1md);
-
-static int RSA_padding_check_PKCS1_OAEP(unsigned char *to, int to_len,
-                                  unsigned char *from, int from_len,
-                                  int rsa_len,
-                                  unsigned char *param, int param_len) {
-    size_t out_len = 0;
-    int ret;
-    int res;
-
-    ret = RSA_padding_check_PKCS1_OAEP_mgf1(to, &out_len, to_len, from, from_len, param, param_len, NULL, NULL);
-    if(!ret) {
-        return(-1);
-    }
-    XMLSEC_SAFE_CAST_SIZE_T_TO_INT(out_len, res, return(-1), NULL);
-    return(res);
-}
-
-
-int RSA_padding_add_PKCS1_OAEP(uint8_t *to, size_t to_len,
-                                              const uint8_t *from,
-                                              size_t from_len,
-                                              const uint8_t *param,
-                                              size_t param_len) {
-    return RSA_padding_add_PKCS1_OAEP_mgf1(to, to_len, from, from_len, param, param_len, NULL, NULL);
-}
-#endif /* OPENSSL_IS_BORINGSSL */
-
-
 
 /**************************************************************************
  *
@@ -548,6 +510,7 @@ xmlSecOpenSSLRsaPkcs1Process(xmlSecTransformPtr transform) {
     return(0);
 }
 
+#ifndef XMLSEC_OPENSSL_NO_RSA_OAEP
 /**************************************************************************
  *
  * Internal OpenSSL RSA OAEP CTX
@@ -558,8 +521,12 @@ typedef struct _xmlSecOpenSSLRsaOaepCtx         xmlSecOpenSSLRsaOaepCtx,
 struct _xmlSecOpenSSLRsaOaepCtx {
 #ifndef XMLSEC_OPENSSL_API_300
     EVP_PKEY*           pKey;
+    const EVP_MD*       md;
+    const EVP_MD*       mgf1md;
 #else /* XMLSEC_OPENSSL_API_300 */
     EVP_PKEY_CTX*       pKeyCtx;
+    const char*         mdName;
+    const char*         mgf1mdName;
     int                 paramsInitialized;
 #endif /* XMLSEC_OPENSSL_API_300 */
     xmlSecSize          keySize;
@@ -568,7 +535,7 @@ struct _xmlSecOpenSSLRsaOaepCtx {
 
 /*********************************************************************
  *
- * RSA OAEP key transport transform
+ * RSA OAEP key transport transform (both XMLEnc 1.0 and XMLEnc 1.1)
  *
  ********************************************************************/
 XMLSEC_TRANSFORM_DECLARE(OpenSSLRsaOaep, xmlSecOpenSSLRsaOaepCtx)
@@ -588,6 +555,8 @@ static int      xmlSecOpenSSLRsaOaepExecute                     (xmlSecTransform
                                                                  xmlSecTransformCtxPtr transformCtx);
 static int      xmlSecOpenSSLRsaOaepProcess                     (xmlSecTransformPtr transform);
 
+static int      xmlSecOpenSSLRsaOaepCheckId                     (xmlSecTransformPtr transform);
+
 static xmlSecTransformKlass xmlSecOpenSSLRsaOaepKlass = {
     /* klass/object sizes */
     sizeof(xmlSecTransformKlass),               /* xmlSecSize klassSize */
@@ -618,7 +587,7 @@ static xmlSecTransformKlass xmlSecOpenSSLRsaOaepKlass = {
 /**
  * xmlSecOpenSSLTransformRsaOaepGetKlass:
  *
- * The RSA-OAEP key transport transform klass.
+ * The RSA-OAEP key transport transform klass (XMLEnc 1.0).
  *
  * Returns: RSA-OAEP key transport transform klass.
  */
@@ -627,6 +596,57 @@ xmlSecOpenSSLTransformRsaOaepGetKlass(void) {
     return(&xmlSecOpenSSLRsaOaepKlass);
 }
 
+static xmlSecTransformKlass xmlSecOpenSSLRsaOaepEnc11Klass = {
+    /* klass/object sizes */
+    sizeof(xmlSecTransformKlass),               /* xmlSecSize klassSize */
+    xmlSecOpenSSLRsaOaepSize,                   /* xmlSecSize objSize */
+
+    xmlSecNameRsaOaepEnc11,                     /* const xmlChar* name; */
+    xmlSecHrefRsaOaepEnc11,                     /* const xmlChar* href; */
+    xmlSecTransformUsageEncryptionMethod,       /* xmlSecAlgorithmUsage usage; */
+
+    xmlSecOpenSSLRsaOaepInitialize,             /* xmlSecTransformInitializeMethod initialize; */
+    xmlSecOpenSSLRsaOaepFinalize,               /* xmlSecTransformFinalizeMethod finalize; */
+    xmlSecOpenSSLRsaOaepNodeRead,               /* xmlSecTransformNodeReadMethod readNode; */
+    NULL,                                       /* xmlSecTransformNodeWriteMethod writeNode; */
+    xmlSecOpenSSLRsaOaepSetKeyReq,              /* xmlSecTransformSetKeyMethod setKeyReq; */
+    xmlSecOpenSSLRsaOaepSetKey,                 /* xmlSecTransformSetKeyMethod setKey; */
+    NULL,                                       /* xmlSecTransformValidateMethod validate; */
+    xmlSecTransformDefaultGetDataType,          /* xmlSecTransformGetDataTypeMethod getDataType; */
+    xmlSecTransformDefaultPushBin,              /* xmlSecTransformPushBinMethod pushBin; */
+    xmlSecTransformDefaultPopBin,               /* xmlSecTransformPopBinMethod popBin; */
+    NULL,                                       /* xmlSecTransformPushXmlMethod pushXml; */
+    NULL,                                       /* xmlSecTransformPopXmlMethod popXml; */
+    xmlSecOpenSSLRsaOaepExecute,                /* xmlSecTransformExecuteMethod execute; */
+
+    NULL,                                       /* void* reserved0; */
+    NULL,                                       /* void* reserved1; */
+};
+
+/**
+ * xmlSecOpenSSLTransformRsaOaepEnc11GetKlass:
+ *
+ * The RSA-OAEP key transport transform klass (XMLEnc 1.1).
+ *
+ * Returns: RSA-OAEP key transport transform klass.
+ */
+xmlSecTransformId
+xmlSecOpenSSLTransformRsaOaepEnc11GetKlass(void) {
+    return(&xmlSecOpenSSLRsaOaepEnc11Klass);
+}
+
+static int
+xmlSecOpenSSLRsaOaepCheckId(xmlSecTransformPtr transform) {
+    if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformRsaOaepId)) {
+        return(1);
+    } else if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformRsaOaepEnc11Id)) {
+        return(1);
+    }
+
+    /* not found */
+    return(0);
+}
+
 #ifndef XMLSEC_OPENSSL_API_300
 
 static int
@@ -663,9 +683,10 @@ xmlSecOpenSSLRsaOaepSetKeyImpl(xmlSecOpenSSLRsaOaepCtxPtr ctx, EVP_PKEY* pKey,
 static int
 xmlSecOpenSSLRsaOaepProcessImpl(xmlSecOpenSSLRsaOaepCtxPtr ctx, const xmlSecByte* inBuf, xmlSecSize inSize,
                             xmlSecByte* outBuf, xmlSecSize* outSize, int encrypt) {
-    xmlSecSize paramsSize;
+    xmlSecByte* oaepLabel;
+    xmlSecSize oaepLabelSize;
     RSA* rsa;
-    int inLen;
+    int inLen, keyLen, oaepLabelLen;
     int ret;
 
     xmlSecAssert2(ctx != NULL, -1);
@@ -679,22 +700,16 @@ xmlSecOpenSSLRsaOaepProcessImpl(xmlSecOpenSSLRsaOaepCtxPtr ctx, const xmlSecByte
     rsa = EVP_PKEY_get0_RSA(ctx->pKey);
     xmlSecAssert2(rsa != NULL, -1);
 
-    paramsSize = xmlSecBufferGetSize(&(ctx->oaepParams));
-    XMLSEC_SAFE_CAST_SIZE_TO_INT(inSize, inLen, return(-1), NULL);
-    if((encrypt != 0) && (paramsSize == 0)) {
-        /* encode w/o OAEPParams --> simple */
-        ret = RSA_public_encrypt(inLen, inBuf, outBuf, rsa, RSA_PKCS1_OAEP_PADDING);
-        if(ret <= 0) {
-            xmlSecOpenSSLError("RSA_public_encrypt(RSA_PKCS1_OAEP_PADDING)", NULL);
-            return(-1);
-        }
-    } else if((encrypt != 0) && (paramsSize != 0)) {
-        xmlSecBuffer tmp;
-        int keyLen, paramLen;
+    oaepLabel = xmlSecBufferGetData(&(ctx->oaepParams));
+    oaepLabelSize = xmlSecBufferGetSize(&(ctx->oaepParams));
 
-        xmlSecAssert2(xmlSecBufferGetData(&(ctx->oaepParams)) != NULL, -1);
-        XMLSEC_SAFE_CAST_SIZE_TO_INT(ctx->keySize, keyLen, return(-1), NULL);
-        XMLSEC_SAFE_CAST_SIZE_TO_INT(paramsSize, paramLen, return(-1), NULL);
+    XMLSEC_SAFE_CAST_SIZE_TO_INT(inSize, inLen, return(-1), NULL);
+    XMLSEC_SAFE_CAST_SIZE_TO_INT(ctx->keySize, keyLen, return(-1), NULL);
+    XMLSEC_SAFE_CAST_SIZE_TO_INT(oaepLabelSize, oaepLabelLen, return(-1), NULL);
+
+    if(encrypt != 0) {
+        /* encrypt */
+        xmlSecBuffer tmp;
 
         /* allocate space for temp buffer */
         ret = xmlSecBufferInitialize(&tmp, ctx->keySize);
@@ -705,10 +720,13 @@ xmlSecOpenSSLRsaOaepProcessImpl(xmlSecOpenSSLRsaOaepCtxPtr ctx, const xmlSecByte
         }
 
         /* add padding */
-        ret = RSA_padding_add_PKCS1_OAEP(xmlSecBufferGetData(&tmp), keyLen,
-            inBuf, inLen , xmlSecBufferGetData(&(ctx->oaepParams)), paramLen);
+        ret = RSA_padding_add_PKCS1_OAEP_mgf1(
+            xmlSecBufferGetData(&tmp), keyLen,
+            inBuf, inLen,
+            oaepLabel, oaepLabelLen,
+            ctx->md, ctx->mgf1md);
         if(ret != 1) {
-            xmlSecOpenSSLError("RSA_padding_add_PKCS1_OAEP", NULL);
+            xmlSecOpenSSLError("RSA_padding_add_PKCS1_OAEP_mgf1", NULL);
             xmlSecBufferFinalize(&tmp);
             return(-1);
         }
@@ -722,19 +740,13 @@ xmlSecOpenSSLRsaOaepProcessImpl(xmlSecOpenSSLRsaOaepCtxPtr ctx, const xmlSecByte
             return(-1);
         }
         xmlSecBufferFinalize(&tmp);
-    } else if((encrypt == 0) && (paramsSize == 0)) {
-        ret = RSA_private_decrypt(inLen, inBuf, outBuf, rsa, RSA_PKCS1_OAEP_PADDING);
-        if(ret <= 0) {
-            xmlSecOpenSSLError("RSA_private_decrypt(RSA_PKCS1_OAEP_PADDING)", NULL);
-            return(-1);
-        }
-    } else if((encrypt == 0) && (paramsSize != 0)) {
-        BIGNUM * bn;
-        int outLen, keyLen, paramLen;
 
-        xmlSecAssert2(xmlSecBufferGetData(&(ctx->oaepParams)) != NULL, -1);
-        XMLSEC_SAFE_CAST_SIZE_TO_INT(ctx->keySize, keyLen, return(-1), NULL);
-        XMLSEC_SAFE_CAST_SIZE_TO_INT(paramsSize, paramLen, return(-1), NULL);
+        /* success */
+        XMLSEC_SAFE_CAST_INT_TO_SIZE(ret, (*outSize), return(-1), NULL);
+    } else {
+        /* decrypt */
+        BIGNUM * bn;
+        int outLen;
 
         ret = RSA_private_decrypt(inLen, inBuf, outBuf, rsa, RSA_NO_PADDING);
         if(ret <= 0) {
@@ -743,7 +755,6 @@ xmlSecOpenSSLRsaOaepProcessImpl(xmlSecOpenSSLRsaOaepCtxPtr ctx, const xmlSecByte
         }
         outLen = ret;
 
-#ifndef OPENSSL_IS_BORINGSSL
         /*
          * the private decrypt w/o padding adds '0's at the beginning.
          * it's not clear for me can I simply skip all '0's from the
@@ -771,22 +782,21 @@ xmlSecOpenSSLRsaOaepProcessImpl(xmlSecOpenSSLRsaOaepCtxPtr ctx, const xmlSecByte
         }
         outLen = ret;
         BN_clear_free(bn);
-#endif /* OPENSSL_IS_BORINGSSL */
 
-        ret = RSA_padding_check_PKCS1_OAEP(outBuf, outLen, outBuf, outLen, keyLen,
-            xmlSecBufferGetData(&(ctx->oaepParams)), paramLen);
+        ret = RSA_padding_check_PKCS1_OAEP_mgf1(
+            outBuf, outLen, outBuf, outLen, keyLen,
+            oaepLabel, oaepLabelLen,
+            ctx->md, ctx->mgf1md);
         if(ret < 0) {
-            xmlSecOpenSSLError("RSA_padding_check_PKCS1_OAEP",  NULL);
+            xmlSecOpenSSLError("RSA_padding_check_PKCS1_OAEP_mgf1",  NULL);
             return(-1);
         }
-    } else {
-        xmlSecInternalError3("Impossible to be here",  NULL,
-            "encrypt=%d; paramsSize=" XMLSEC_SIZE_FMT, encrypt, paramsSize);
-        return(-1);
+
+        /* success */
+        XMLSEC_SAFE_CAST_INT_TO_SIZE(ret, (*outSize), return(-1), NULL);
     }
 
     /* success */
-    XMLSEC_SAFE_CAST_INT_TO_SIZE(ret, (*outSize), return(-1), NULL);
     return(0);
 }
 
@@ -839,60 +849,77 @@ xmlSecOpenSSLRsaOaepSetKeyImpl(xmlSecOpenSSLRsaOaepCtxPtr ctx, EVP_PKEY* pKey,
     return(0);
 }
 
+// We can put all the params into one OSSL_PARAM array and setup everything at-once.
+// However, in OpenSSL <= 3.0.7 there is a bug that mixes OAEP digest and
+// OAEP MGf1 digest (https://pullanswer.com/questions/mgf1-digest-not-set-correctly-when-configuring-rsa-evp_pkey_ctx-with-ossl_params)
+// so we do one param at a time.
 static int
-xmlSecOpenSSSLRsaOaepSetParams(EVP_PKEY_CTX* pKeyCtx,
-                            const xmlSecByte* paramsBuf, xmlSecSize paramsSize) {
-    OSSL_PARAM_BLD* param_bld = NULL;
-    OSSL_PARAM* params = NULL;
+xmlSecOpenSSSLRsaOaepSetParamsIfNeeded(xmlSecOpenSSLRsaOaepCtxPtr ctx) {
+    xmlSecByte* label;
+    xmlSecSize labelSize;
     int ret;
     int res = -1;
 
-    xmlSecAssert2(pKeyCtx != NULL, -1);
-    xmlSecAssert2(paramsBuf != NULL, -1)
-    xmlSecAssert2(paramsSize > 0, -1);
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->pKeyCtx != NULL, -1);
 
-    param_bld = OSSL_PARAM_BLD_new();
-    if(param_bld == NULL) {
-        xmlSecOpenSSLError("OSSL_PARAM_BLD_new", NULL);
-        goto done;
+    /* check if we already initialized oaep params */
+    if(ctx->paramsInitialized != 0) {
+        return(0);
     }
 
-    ret = OSSL_PARAM_BLD_push_octet_string(param_bld, OSSL_ASYM_CIPHER_PARAM_OAEP_LABEL,
-        paramsBuf, paramsSize);
-    if(ret != 1) {
-        xmlSecOpenSSLError("OSSL_PARAM_BLD_push_octet_string(label)", NULL);
-        goto done;
-     }
+    /* OAEP label */
+    label = xmlSecBufferGetData(&(ctx->oaepParams));
+    labelSize = xmlSecBufferGetSize(&(ctx->oaepParams));
+    if((label != NULL) && (labelSize > 0)) {
+        OSSL_PARAM params[2];
+        params[0] = OSSL_PARAM_construct_octet_string(OSSL_ASYM_CIPHER_PARAM_OAEP_LABEL, label, labelSize);
+        params[1] = OSSL_PARAM_construct_end();
 
-     params = OSSL_PARAM_BLD_to_param(param_bld);
-     if(params == NULL) {
-        xmlSecOpenSSLError("OSSL_PARAM_BLD_to_param", NULL);
-        goto done;
-     }
+        ret = EVP_PKEY_CTX_set_params(ctx->pKeyCtx, params);
+        if(ret <= 0) {
+            xmlSecOpenSSLError("EVP_PKEY_CTX_set_params", NULL);
+            goto done;
+        }
+    }
 
-     ret = EVP_PKEY_CTX_set_params(pKeyCtx, params);
-     if(ret <= 0) {
-        xmlSecOpenSSLError("EVP_PKEY_CTX_set_params", NULL);
-        goto done;
-     }
+    /* Digest */
+    if(ctx->mdName != NULL) {
+        OSSL_PARAM params[2];
+        params[0] = OSSL_PARAM_construct_utf8_string(OSSL_ASYM_CIPHER_PARAM_OAEP_DIGEST, (char*)ctx->mdName, 0);
+        params[1] = OSSL_PARAM_construct_end();
 
-     /* success */
-     res = 0;
+        ret = EVP_PKEY_CTX_set_params(ctx->pKeyCtx, params);
+        if(ret <= 0) {
+            xmlSecOpenSSLError("EVP_PKEY_CTX_set_params", NULL);
+            goto done;
+        }
+    }
+
+    /* MGF1 digest */
+    if(ctx->mgf1mdName != NULL) {
+        OSSL_PARAM params[2];
+        params[0] = OSSL_PARAM_construct_utf8_string(OSSL_ASYM_CIPHER_PARAM_MGF1_DIGEST, (char*)ctx->mgf1mdName, 0);
+        params[1] = OSSL_PARAM_construct_end();
+
+        ret = EVP_PKEY_CTX_set_params(ctx->pKeyCtx, params);
+        if(ret <= 0) {
+            xmlSecOpenSSLError("EVP_PKEY_CTX_set_params", NULL);
+            goto done;
+        }
+    }
+
+    /* success */
+    ctx->paramsInitialized = 1;
+    res = 0;
 
 done:
-    if(params != NULL) {
-        OSSL_PARAM_free(params);
-    }
-    if(param_bld != NULL) {
-        OSSL_PARAM_BLD_free(param_bld);
-    }
     return(res);
 }
 
 static int
 xmlSecOpenSSLRsaOaepProcessImpl(xmlSecOpenSSLRsaOaepCtxPtr ctx, const xmlSecByte* inBuf, xmlSecSize inSize,
                             xmlSecByte* outBuf, xmlSecSize* outSize, int encrypt) {
-    xmlSecSize paramsSize;
     size_t outSizeT;
     int ret;
 
@@ -903,15 +930,10 @@ xmlSecOpenSSLRsaOaepProcessImpl(xmlSecOpenSSLRsaOaepCtxPtr ctx, const xmlSecByte
     xmlSecAssert2(outBuf != NULL, -1);
     xmlSecAssert2(outSize != NULL, -1);
 
-    paramsSize = xmlSecBufferGetSize(&(ctx->oaepParams));
-    if((paramsSize > 0) && (ctx->paramsInitialized == 0)){
-        ret = xmlSecOpenSSSLRsaOaepSetParams(ctx->pKeyCtx,
-            xmlSecBufferGetData(&(ctx->oaepParams)), paramsSize);
-        if(ret != 0) {
-            xmlSecInternalError("xmlSecOpenSSSLRsaOaepSetParams", NULL);
-            return(-1);
-        }
-        ctx->paramsInitialized = 1;
+    ret = xmlSecOpenSSSLRsaOaepSetParamsIfNeeded(ctx);
+    if(ret != 0) {
+        xmlSecInternalError("xmlSecOpenSSSLRsaOaepSetParamsIfNeeded", NULL);
+        return(-1);
     }
 
     outSizeT = (*outSize);
@@ -940,7 +962,7 @@ xmlSecOpenSSLRsaOaepInitialize(xmlSecTransformPtr transform) {
     xmlSecOpenSSLRsaOaepCtxPtr ctx;
     int ret;
 
-    xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformRsaOaepId), -1);
+    xmlSecAssert2(xmlSecOpenSSLRsaOaepCheckId(transform), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecOpenSSLRsaOaepSize), -1);
 
     ctx = xmlSecOpenSSLRsaOaepGetCtx(transform);
@@ -961,7 +983,7 @@ static void
 xmlSecOpenSSLRsaOaepFinalize(xmlSecTransformPtr transform) {
     xmlSecOpenSSLRsaOaepCtxPtr ctx;
 
-    xmlSecAssert(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformRsaOaepId));
+    xmlSecAssert(xmlSecOpenSSLRsaOaepCheckId(transform));
     xmlSecAssert(xmlSecTransformCheckSize(transform, xmlSecOpenSSLRsaOaepSize));
 
     ctx = xmlSecOpenSSLRsaOaepGetCtx(transform);
@@ -981,14 +1003,27 @@ xmlSecOpenSSLRsaOaepFinalize(xmlSecTransformPtr transform) {
     memset(ctx, 0, sizeof(xmlSecOpenSSLRsaOaepCtx));
 }
 
+/* small helper macros to reduce clutter in the code */
+#ifndef XMLSEC_OPENSSL_API_300
+#define XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, digestVal, digestNameVal) \
+    (ctx)->md = (digestVal)
+#define XMLSEC_OPENSSL_OAEP_MGF1_DIGEST_SETUP(ctx, digestVal, digestNameVal) \
+    (ctx)->mgf1md = (digestVal)
+#else /* XMLSEC_OPENSSL_API_300 */
+#define XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, digestVal, digestNameVal) \
+    (ctx)->mdName = (digestNameVal)
+#define XMLSEC_OPENSSL_OAEP_MGF1_DIGEST_SETUP(ctx, digestVal, digestNameVal) \
+    (ctx)->mgf1mdName = (digestNameVal)
+#endif /* XMLSEC_OPENSSL_API_300 */
+
 static int
 xmlSecOpenSSLRsaOaepNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
                              xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
     xmlSecOpenSSLRsaOaepCtxPtr ctx;
-    xmlChar* algorithm = NULL;
+    xmlSecTransformRsaOaepParams oaepParams;
     int ret;
 
-    xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformRsaOaepId), -1);
+    xmlSecAssert2(xmlSecOpenSSLRsaOaepCheckId(transform), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecOpenSSLRsaOaepSize), -1);
     xmlSecAssert2(node != NULL, -1);
     UNREFERENCED_PARAMETER(transformCtx);
@@ -997,24 +1032,145 @@ xmlSecOpenSSLRsaOaepNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
     xmlSecAssert2(ctx != NULL, -1);
     xmlSecAssert2(xmlSecBufferGetSize(&(ctx->oaepParams)) == 0, -1);
 
-    ret = xmlSecTransformRsaOaepReadParams(node, &(ctx->oaepParams), &algorithm);
+    ret = xmlSecTransformRsaOaepParamsInitialize(&oaepParams);
     if (ret < 0) {
-        xmlSecInternalError("xmlSecTransformRsaOaepReadParams",
+        xmlSecInternalError("xmlSecTransformRsaOaepParamsInitialize",
             xmlSecTransformGetName(transform));
         return(-1);
     }
 
-    /* for now we support only sha1 */
-    if ((algorithm != NULL) && (xmlStrcmp(algorithm, xmlSecHrefSha1) != 0)) {
-        xmlSecInvalidTransfromError2(transform,
-            "digest algorithm=\"%s\" is not supported for rsa/oaep",
-            xmlSecErrorsSafeString(algorithm));
-        xmlFree(algorithm);
+    ret = xmlSecTransformRsaOaepParamsRead(&oaepParams, node);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecTransformRsaOaepParamsRead",
+            xmlSecTransformGetName(transform));
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
         return(-1);
     }
-    xmlFree(algorithm);
 
-    /* done */
+    /* digest algorithm */
+    if(oaepParams.digestAlgorithm == NULL) {
+#ifndef XMLSEC_NO_SHA1
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_sha1(), OSSL_DIGEST_NAME_SHA1);
+#else  /* XMLSEC_NO_SHA1 */
+        xmlSecOtherError(XMLSEC_ERRORS_R_DISABLED, NULL, "No OAEP digest algorithm is specified and the default SHA1 digest is disabled");
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
+        return(-1);
+#endif /* XMLSEC_NO_SHA1 */
+    } else
+#ifndef XMLSEC_NO_MD5
+    if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefMd5) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_md5(), OSSL_DIGEST_NAME_MD5);
+    } else
+#endif /* XMLSEC_NO_MD5 */
+
+#ifndef XMLSEC_NO_RIPEMD160
+    if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefRipemd160) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_ripemd160(), OSSL_DIGEST_NAME_RIPEMD160);
+    } else
+#endif /* XMLSEC_NO_RIPEMD160 */
+
+#ifndef XMLSEC_NO_SHA1
+    if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha1) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_sha1(), OSSL_DIGEST_NAME_SHA1);
+    } else
+#endif /* XMLSEC_NO_SHA1 */
+
+#ifndef XMLSEC_NO_SHA224
+    if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha224) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_sha224(), OSSL_DIGEST_NAME_SHA2_224);
+    } else
+#endif /* XMLSEC_NO_SHA224 */
+
+#ifndef XMLSEC_NO_SHA256
+    if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha256) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_sha256(), OSSL_DIGEST_NAME_SHA2_256);
+    } else
+#endif /* XMLSEC_NO_SHA256 */
+
+#ifndef XMLSEC_NO_SHA384
+    if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha384) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_sha384(), OSSL_DIGEST_NAME_SHA2_384);
+    } else
+#endif /* XMLSEC_NO_SHA384 */
+
+#ifndef XMLSEC_NO_SHA512
+    if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha512) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_sha512(), OSSL_DIGEST_NAME_SHA2_512);
+    } else
+#endif /* XMLSEC_NO_SHA512 */
+
+#ifndef XMLSEC_NO_SHA3
+    if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha3_224) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_sha3_224(), OSSL_DIGEST_NAME_SHA3_224);
+    } else if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha3_256) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_sha3_256(), OSSL_DIGEST_NAME_SHA3_256);
+    } else if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha3_384) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_sha3_384(), OSSL_DIGEST_NAME_SHA3_384);
+    } else if(xmlStrcmp(oaepParams.digestAlgorithm, xmlSecHrefSha3_512) == 0) {
+        XMLSEC_OPENSSL_OAEP_DIGEST_SETUP(ctx, EVP_sha3_512(), OSSL_DIGEST_NAME_SHA3_512);
+    } else
+#endif /* XMLSEC_NO_SHA3 */
+
+    {
+       xmlSecInvalidTransfromError2(transform,
+            "digest algorithm=\"%s\" is not supported for rsa/oaep",
+            xmlSecErrorsSafeString(oaepParams.digestAlgorithm));
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
+        return(-1);
+    }
+
+    /* mgf1 algorithm */
+    if(oaepParams.mgf1DigestAlgorithm == NULL) {
+#ifndef XMLSEC_NO_SHA1
+        XMLSEC_OPENSSL_OAEP_MGF1_DIGEST_SETUP(ctx, EVP_sha1(), OSSL_DIGEST_NAME_SHA1);
+#else  /* XMLSEC_NO_SHA1 */
+        xmlSecOtherError(XMLSEC_ERRORS_R_DISABLED, NULL, "No OAEP mgf1 digest algorithm is specified and the default SHA1 digest is disabled");
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
+        return(-1);
+#endif /* XMLSEC_NO_SHA1 */
+    } else
+#ifndef XMLSEC_NO_SHA1
+    if(xmlStrcmp(oaepParams.mgf1DigestAlgorithm, xmlSecHrefMgf1Sha1) == 0) {
+        XMLSEC_OPENSSL_OAEP_MGF1_DIGEST_SETUP(ctx, EVP_sha1(), OSSL_DIGEST_NAME_SHA1);
+    } else
+#endif /* XMLSEC_NO_SHA1 */
+
+#ifndef XMLSEC_NO_SHA224
+    if(xmlStrcmp(oaepParams.mgf1DigestAlgorithm, xmlSecHrefMgf1Sha224) == 0) {
+        XMLSEC_OPENSSL_OAEP_MGF1_DIGEST_SETUP(ctx, EVP_sha224(), OSSL_DIGEST_NAME_SHA2_224);
+    } else
+#endif /* XMLSEC_NO_SHA224 */
+
+#ifndef XMLSEC_NO_SHA256
+    if(xmlStrcmp(oaepParams.mgf1DigestAlgorithm, xmlSecHrefMgf1Sha256) == 0) {
+        XMLSEC_OPENSSL_OAEP_MGF1_DIGEST_SETUP(ctx, EVP_sha256(), OSSL_DIGEST_NAME_SHA2_256);
+    } else
+#endif /* XMLSEC_NO_SHA256 */
+
+#ifndef XMLSEC_NO_SHA384
+    if(xmlStrcmp(oaepParams.mgf1DigestAlgorithm, xmlSecHrefMgf1Sha384) == 0) {
+        XMLSEC_OPENSSL_OAEP_MGF1_DIGEST_SETUP(ctx, EVP_sha384(), OSSL_DIGEST_NAME_SHA2_384);
+    } else
+#endif /* XMLSEC_NO_SHA384 */
+
+#ifndef XMLSEC_NO_SHA512
+    if(xmlStrcmp(oaepParams.mgf1DigestAlgorithm, xmlSecHrefMgf1Sha512) == 0) {
+        XMLSEC_OPENSSL_OAEP_MGF1_DIGEST_SETUP(ctx, EVP_sha512(), OSSL_DIGEST_NAME_SHA2_512);
+    } else
+#endif /* XMLSEC_NO_SHA512 */
+    {
+       xmlSecInvalidTransfromError2(transform,
+            "mgf1 digest algorithm=\"%s\" is not supported for rsa/oaep",
+            xmlSecErrorsSafeString(oaepParams.mgf1DigestAlgorithm));
+        xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
+        return(-1);
+    }
+
+    /* put oaep params buffer into ctx */
+    xmlSecBufferSwap(&(oaepParams.oaepParams), &(ctx->oaepParams));
+
+    /* cleanup */
+    xmlSecTransformRsaOaepParamsFinalize(&oaepParams);
     return(0);
 }
 
@@ -1022,7 +1178,7 @@ static int
 xmlSecOpenSSLRsaOaepSetKeyReq(xmlSecTransformPtr transform,  xmlSecKeyReqPtr keyReq) {
     xmlSecOpenSSLRsaOaepCtxPtr ctx;
 
-    xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformRsaOaepId), -1);
+    xmlSecAssert2(xmlSecOpenSSLRsaOaepCheckId(transform), -1);
     xmlSecAssert2((transform->operation == xmlSecTransformOperationEncrypt) || (transform->operation == xmlSecTransformOperationDecrypt), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecOpenSSLRsaOaepSize), -1);
     xmlSecAssert2(keyReq != NULL, -1);
@@ -1049,7 +1205,7 @@ xmlSecOpenSSLRsaOaepSetKey(xmlSecTransformPtr transform, xmlSecKeyPtr key) {
     int encrypt;
     int ret;
 
-    xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformRsaOaepId), -1);
+    xmlSecAssert2(xmlSecOpenSSLRsaOaepCheckId(transform), -1);
     xmlSecAssert2((transform->operation == xmlSecTransformOperationEncrypt) || (transform->operation == xmlSecTransformOperationDecrypt), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecOpenSSLRsaOaepSize), -1);
     xmlSecAssert2(key != NULL, -1);
@@ -1096,7 +1252,7 @@ xmlSecOpenSSLRsaOaepExecute(xmlSecTransformPtr transform, int last,
     xmlSecOpenSSLRsaOaepCtxPtr ctx;
     int ret;
 
-    xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformRsaOaepId), -1);
+    xmlSecAssert2(xmlSecOpenSSLRsaOaepCheckId(transform), -1);
     xmlSecAssert2((transform->operation == xmlSecTransformOperationEncrypt) || (transform->operation == xmlSecTransformOperationDecrypt), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecOpenSSLRsaOaepSize), -1);
     UNREFERENCED_PARAMETER(transformCtx);
@@ -1137,7 +1293,7 @@ xmlSecOpenSSLRsaOaepProcess(xmlSecTransformPtr transform) {
     int encrypt;
     int ret;
 
-    xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformRsaOaepId), -1);
+    xmlSecAssert2(xmlSecOpenSSLRsaOaepCheckId(transform), -1);
     xmlSecAssert2((transform->operation == xmlSecTransformOperationEncrypt) || (transform->operation == xmlSecTransformOperationDecrypt), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecOpenSSLRsaOaepSize), -1);
 
@@ -1211,6 +1367,7 @@ xmlSecOpenSSLRsaOaepProcess(xmlSecTransformPtr transform) {
 
     return(0);
 }
+#endif /* XMLSEC_OPENSSL_NO_RSA_OAEP */
+
 
 #endif /* XMLSEC_NO_RSA */
-
