@@ -552,6 +552,7 @@ xmlSecOpenSSLKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
 
     /* crts */
     if(ctxSrc->certsList != NULL) {
+#ifndef XMLSEC_OPENSSL_NO_DEEP_COPY
 #ifndef XMLSEC_OPENSSL_API_300
         ctxDst->certsList = sk_X509_deep_copy(ctxSrc->certsList, (sk_X509_copyfunc)X509_dup, X509_free);
 #else  /* XMLSEC_OPENSSL_API_300 */
@@ -561,10 +562,41 @@ xmlSecOpenSSLKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
             xmlSecOpenSSLError("sk_X509_deep_copy", xmlSecKeyDataGetName(dst));
             return(-1);
         }
+#else /* XMLSEC_OPENSSL_NO_DEEP_COPY */
+        int size, ii;
+        X509* certSrc;
+        X509* certDst;
+        int ret;
+
+        ctxDst->certsList = sk_X509_new_null();
+        if(ctxDst->certsList == NULL) {
+            xmlSecOpenSSLError("sk_X509_new_null", xmlSecKeyDataGetName(dst));
+            return(-1);
+        }
+        size = sk_X509_num(ctxSrc->certsList);
+        for(ii = 0; ii < size; ++ii) {
+            certSrc = sk_X509_value(ctxSrc->certsList, ii);
+            if(certSrc == NULL) {
+                continue;
+            }
+            certDst = X509_dup(certSrc);
+            if(certDst == NULL) {
+                xmlSecOpenSSLError("X509_dup", xmlSecKeyDataGetName(dst));
+                return(-1);
+            }
+            ret = sk_X509_push(ctxDst->certsList, certDst);
+            if(ret <= 0) {
+                xmlSecOpenSSLError("sk_X509_push", NULL);
+                X509_free(certDst);
+                return(-1);
+            }
+        }
+#endif /* XMLSEC_OPENSSL_NO_DEEP_COPY */
     }
 
     /* crls */
     if(ctxSrc->crlsList != NULL) {
+#ifndef XMLSEC_OPENSSL_NO_DEEP_COPY
 #ifndef XMLSEC_OPENSSL_API_300
         ctxDst->crlsList = sk_X509_CRL_deep_copy(ctxSrc->crlsList, (sk_X509_CRL_copyfunc)X509_CRL_dup, X509_CRL_free);
 #else  /* XMLSEC_OPENSSL_API_300 */
@@ -574,6 +606,36 @@ xmlSecOpenSSLKeyDataX509Duplicate(xmlSecKeyDataPtr dst, xmlSecKeyDataPtr src) {
             xmlSecOpenSSLError("sk_X509_CRL_deep_copy", xmlSecKeyDataGetName(dst));
             return(-1);
         }
+#else /* XMLSEC_OPENSSL_NO_DEEP_COPY */
+        int size, ii;
+        X509_CRL* crlSrc;
+        X509_CRL* crlDst;
+        int ret;
+
+        ctxDst->crlsList = sk_X509_CRL_new_null();
+        if(ctxDst->crlsList == NULL) {
+            xmlSecOpenSSLError("sk_X509_CRL_new_null", xmlSecKeyDataGetName(dst));
+            return(-1);
+        }
+        size = sk_X509_CRL_num(ctxSrc->crlsList);
+        for(ii = 0; ii < size; ++ii) {
+            crlSrc = sk_X509_CRL_value(ctxSrc->crlsList, ii);
+            if(crlSrc == NULL) {
+                continue;
+            }
+            crlDst = X509_CRL_dup(crlSrc);
+            if(crlDst == NULL) {
+                xmlSecOpenSSLError("X509_CRL_dup", xmlSecKeyDataGetName(dst));
+                return(-1);
+            }
+            ret = sk_X509_CRL_push(ctxDst->crlsList, crlDst);
+            if(ret <= 0) {
+                xmlSecOpenSSLError("sk_X509_CRL_push", NULL);
+                X509_CRL_free(crlDst);
+                return(-1);
+            }
+        }
+#endif /* XMLSEC_OPENSSL_NO_DEEP_COPY */
     }
 
     /* keyCert: should be in the same position in certsList after copy */
@@ -1393,42 +1455,45 @@ my_timegm(struct tm *t) {
 
 #endif /* HAVE_TIMEGM */
 
-#if !defined(OPENSSL_IS_BORINGSSL)
+#ifndef XMLSEC_OPENSSL_NO_ASN1_TIME_TO_TM
 
-time_t
-xmlSecOpenSSLX509Asn1TimeToTime(const ASN1_TIME * t) {
+int
+xmlSecOpenSSLX509Asn1TimeToTime(const ASN1_TIME * t, time_t * res) {
     struct tm tm;
     int ret;
 
-    xmlSecAssert2(t != NULL, 0);
+    xmlSecAssert2(t != NULL, -1);
+    xmlSecAssert2(res != NULL, -1);
 
     if(!ASN1_TIME_check(t)) {
         xmlSecOpenSSLError("ASN1_TIME_check", NULL);
-        return(0);
+        return(-1);
     }
 
     memset(&tm, 0, sizeof(tm));
     ret = ASN1_TIME_to_tm(t, &tm);
     if(ret != 1) {
         xmlSecOpenSSLError("ASN1_TIME_to_tm", NULL);
-        return(0);
+        return(-1);
     }
 
-    return(timegm(&tm));
+    (*res) = timegm(&tm);
+    return (0);
 }
 
-#else  /* !defined(OPENSSL_IS_BORINGSSL) */
+#else  /* XMLSEC_OPENSSL_NO_ASN1_TIME_TO_TM */
 
-time_t
-xmlSecOpenSSLX509Asn1TimeToTime(ASN1_TIME * t) {
+int
+xmlSecOpenSSLX509Asn1TimeToTime(const ASN1_TIME * t, time_t * res) {
     struct tm tm;
     int offset;
 
-    xmlSecAssert2(t != NULL, 0);
+    xmlSecAssert2(t != NULL, -1);
+    xmlSecAssert2(res != NULL, -1);
 
     if(!ASN1_TIME_check(t)) {
         xmlSecOpenSSLError("ASN1_TIME_check", NULL);
-        return(0);
+        return(-1);
     }
 
     memset(&tm, 0, sizeof(tm));
@@ -1480,9 +1545,11 @@ xmlSecOpenSSLX509Asn1TimeToTime(ASN1_TIME * t) {
         tm.tm_isdst = -1;
     }
 #undef g2
-    return(timegm(&tm) - offset * 60);
+
+    (*res) = (timegm(&tm) - offset * 60);
+    return (0);
 }
-#endif /* !defined(OPENSSL_IS_BORINGSSL) */
+#endif /* XMLSEC_OPENSSL_NO_ASN1_TIME_TO_TM */
 
 /* returns 1 if cert was found and verified and also data was adopted, 0 if not, or negative value if an error occurs */
 static int
@@ -1561,8 +1628,8 @@ xmlSecOpenSSLVerifyAndAdoptX509KeyData(xmlSecKeyPtr key, xmlSecKeyDataPtr data,
 
     /* copy cert not before / not after times from the cert */
     if(X509_get0_notBefore(ctx->keyCert) != NULL) {
-        key->notValidBefore = xmlSecOpenSSLX509Asn1TimeToTime(X509_get0_notBefore(ctx->keyCert));
-        if(key->notValidBefore <= 0) {
+        ret = xmlSecOpenSSLX509Asn1TimeToTime(X509_get0_notBefore(ctx->keyCert), &(key->notValidBefore));
+        if(ret < 0) {
             xmlSecInternalError("xmlSecOpenSSLX509Asn1TimeToTime(notValidBefore)", xmlSecKeyDataGetName(data));
             return(-1);
         }
@@ -1570,8 +1637,8 @@ xmlSecOpenSSLVerifyAndAdoptX509KeyData(xmlSecKeyPtr key, xmlSecKeyDataPtr data,
         key->notValidBefore = 0;
     }
     if(X509_get0_notAfter(ctx->keyCert) != NULL) {
-        key->notValidAfter = xmlSecOpenSSLX509Asn1TimeToTime(X509_get0_notAfter(ctx->keyCert));
-        if(key->notValidAfter <= 0) {
+        ret = xmlSecOpenSSLX509Asn1TimeToTime(X509_get0_notAfter(ctx->keyCert), &(key->notValidAfter));
+        if(ret < 0) {
             xmlSecInternalError("xmlSecOpenSSLX509Asn1TimeToTime(notValidAfter)", xmlSecKeyDataGetName(data));
             return(-1);
         }
